@@ -13,16 +13,19 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: dialog.c,v 1.4 1997/09/03 19:31:07 torsten Exp $ */
+/* $Id: dialog.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#include <stdio.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Toggle.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Label.h>
 #include <X11/StringDefs.h>
-#include <stdio.h>
+
+#include "xaw_incdir/Command.h"
+#include "xaw_incdir/Toggle.h"
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Label.h"
+
 #ifdef MISSING_STDARG_H
 #include <varargs.h>
 #else
@@ -80,7 +83,7 @@ AlertBox(Widget parent, char *msg, XtCallbackProc okProc,
     Widget form, title, okButton = None, cancelButton = None;
     arg_t *arg = XtNew(arg_t);
     int nargs = 0;
-    Arg args[5];
+    Arg args[6];
     
     XtVaGetValues(GetShell(parent), XtNx, &x, XtNy, &y, NULL);
 
@@ -110,23 +113,25 @@ AlertBox(Widget parent, char *msg, XtCallbackProc okProc,
     okButton = XtVaCreateManagedWidget("ok",
 				       commandWidgetClass, form,
 				       XtNfromVert, title,
-				       XtNlabel, "OK",
-    /* XtNaccelerators, accel, */
+				       /* XtNlabel, "OK", */
+                                       /* XtNaccelerators, accel, */
 				       NULL);
     XtAddCallback(okButton, XtNcallback, okCallback, (XtPointer) arg);
 
     if (nokProc != NULL) {
-	cancelButton = XtVaCreateManagedWidget("cancel",
+	cancelButton = XtVaCreateManagedWidget("stopit",
 					       commandWidgetClass, form,
 					       XtNfromVert, title,
 					       XtNfromHoriz, okButton,
-					       XtNlabel, "Cancel",
+					       /* XtNlabel, "Cancel", */
 					       NULL);
 	XtAddCallback(cancelButton, XtNcallback, cancelCallback, (XtPointer) arg);
     }
     AddDestroyCallback(shell, (DestroyCallbackFunc) cancelCallback,
 		       NULL);
     XtPopup(shell, XtGrabExclusive);
+    XtUnmanageChild(form);
+    XMapWindow(XtDisplay(shell), XtWindow(form));
 }
 
 #ifdef MISSING_STDARG_H
