@@ -16,6 +16,11 @@
 /* Drawing with a dynamic pen. Adapted from Paul Haeberli's
    dynadraw.c (for Iris GL),
    see http://www.sgi.com/grafica/dyna/index.html */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#define XtWindow XTWINDOW
+#endif
    
 #include <math.h>
 #include <sys/param.h>
@@ -43,7 +48,7 @@ typedef struct {
 	float	width;
 	float	odelx, odely;
 	float	mass, drag;
-	int	doFinish, finishing;
+	int	doFinish, pressed, finishing;
 	XtIntervalId tid;
 	Widget  w;
 	OpInfo	*opinfo;
@@ -124,17 +129,21 @@ printf("acc = (%f, %f), vel = (%f, %f)\n", l->accx, l->accy, l->velx, l->vely);
 static void
 press(Widget w, LocalInfo *l, XButtonEvent *event, OpInfo *info) 
 {
+        int save;
 	if (info->surface == opWindow)
 		return;
 
 	if ((event->state & (Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask)) != 0)
 		return;
+        if (event->button >= Button3) return;
+        if (event->type == MotionNotify) return;
 
 	l->mass	    = mass;
 	l->drag     = drag;
 	l->width    = width;
 	l->doFinish = doFinish;
 	l->finishing = False;
+	l->pressed = True;
 	l->fixedangle = 0;
 	l->tid	    = 0;
 	l->w	    = w;
@@ -153,15 +162,20 @@ press(Widget w, LocalInfo *l, XButtonEvent *event, OpInfo *info)
 	else
 		l->gc = info->first_gc;
 
+        save = Global.dashnumber;
+        Global.dashnumber = 0;
+        SetCapAndJoin(w, l->gc, JoinRound);
+        Global.dashnumber = save;
+
 	UndoStart(w, info);
 }
 
 static Bool
 dummy_pred(Display *d, XEvent *ev, char *arg)
 {
-	return (ev->type == MotionNotify
-		|| (ev->type == PropertyNotify
-		    && ev->xproperty.atom == dummy_prop));
+	return (ev->type == MotionNotify ||
+		  (ev->type == PropertyNotify
+		     && ev->xproperty.atom == dummy_prop));
 }
 
 static void motion(Widget, LocalInfo *, XMotionEvent *, OpInfo *);
@@ -172,8 +186,6 @@ timeout(XtPointer client_data, XtIntervalId *iid)
 	XEvent ev;
 	XMotionEvent me;
 	LocalInfo *l = (LocalInfo *)client_data;
-	Window root, child;
-	int rootx, rooty;
 
 	XChangeProperty(XtDisplay(l->w), XtWindow(l->w), dummy_prop,
 			XA_ATOM, 32, PropModeReplace,
@@ -183,10 +195,10 @@ timeout(XtPointer client_data, XtIntervalId *iid)
 	   or a MotionNotify */
 
 	XPeekIfEvent(XtDisplay(l->w), &ev, dummy_pred, NULL);
-	
-	if (ev.type == MotionNotify)
-		return;
 
+	if (ev.type == MotionNotify)
+	   return;
+   
 	/* It's a PropertyNotify */
 	l->tid = 0;
 	me.x = l->mousex;
@@ -201,8 +213,11 @@ motion(Widget w, LocalInfo *l, XMotionEvent *event, OpInfo *info)
 {
 	XRectangle	undo;
 
+	if (!l->pressed)
+		return;
 	if (info->surface == opWindow)
 		return;
+        if (l->gc != info->first_gc && l->gc != info->second_gc) return;
 
 	if (l->tid)
 		XtRemoveTimeOut(l->tid), l->tid = 0;
@@ -258,7 +273,7 @@ printf("motion: %d: (%d, %d)\n", event->time, event->x, event->y);
 		l->starty = event->y;
 
 		PwUpdate(w, &undo, False);
-
+	     
 		if (!l->finishing)
 			l->tid = XtAppAddTimeOut(XtWidgetToApplicationContext(w),
 						 XTIMEOUT, timeout, l);
@@ -282,14 +297,22 @@ release(Widget w, LocalInfo *l, XButtonEvent *event, OpInfo *info)
         case Button4:   mask ^= Button4Mask; break;
         case Button5:   mask ^= Button5Mask; break;
         }
+   
+        if ((event->type & PointerMotionMask) == MotionNotify) return;
+
         if ((event->state & mask) != 0)
                 return;
 
+        if (event->button >= Button3) return;
+
 	if (l->tid)
 		XtRemoveTimeOut(l->tid), l->tid = 0;
 
-	if (!l->doFinish)
+	if (!l->doFinish) {
+                l->pressed = False;
 		return;
+	}
+
 	l->finishing = True;
 
 	me.x = event->x;
@@ -299,6 +322,7 @@ release(Widget w, LocalInfo *l, XButtonEvent *event, OpInfo *info)
 		me.time += XTIMEOUT;
 		motion(w, l, &me, info);
 	}
+        l->pressed = False;
 }
 
 /*
@@ -334,8 +358,7 @@ void
 
 	OpAddEventHandler(w, opPixmap, ButtonPressMask, FALSE, (OpEventProc)press, l);
 	OpAddEventHandler(w, opPixmap, ButtonMotionMask, FALSE, (OpEventProc)motion, l);
-	OpAddEventHandler(w, opPixmap, ButtonReleaseMask, FALSE, (OpEventProc)release, l);
-
+	OpAddEventHandler(w, opPixmap, PointerMotionMask|ButtonReleaseMask, FALSE, (OpEventProc)release, l);
 	SetPencilCursor(w);
 
 	if (dummy_prop == None)
@@ -353,6 +376,6 @@ DynPencilRemove(Widget w, LocalInfo *l)
 {
 	OpRemoveEventHandler(w, opPixmap, ButtonPressMask, FALSE, (OpEventProc)press, l);
 	OpRemoveEventHandler(w, opPixmap, ButtonMotionMask, FALSE, (OpEventProc)motion, l);
-	OpRemoveEventHandler(w, opPixmap, ButtonReleaseMask, FALSE, (OpEventProc)release, l);
+	OpRemoveEventHandler(w, opPixmap, PointerMotionMask|ButtonReleaseMask, FALSE, (OpEventProc)release, l);
 	XtFree((XtPointer)l);
 }
