@@ -13,31 +13,33 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: fileName.c,v 1.8 1997/09/03 19:31:07 torsten Exp $ */
+/* $Id: fileName.c,v 1.18 2005/03/20 20:15:32 demailly Exp $ */
 
 #include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
-#include <X11/Xaw/Dialog.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Toggle.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Label.h>
-#include <X11/Xaw/List.h>
-#include <X11/Xaw/AsciiText.h>
-#include <X11/Xaw/Text.h>
-#include <X11/Xaw/Viewport.h>
-#include <X11/Xaw/Scrollbar.h>
+
+#include "xaw_incdir/Dialog.h"
+#include "xaw_incdir/Command.h"
+#include "xaw_incdir/Toggle.h"
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Label.h"
+#include "xaw_incdir/List.h"
+#include "xaw_incdir/AsciiText.h"
+#include "xaw_incdir/Text.h"
+#include "xaw_incdir/Viewport.h"
+#include "xaw_incdir/Scrollbar.h"
 
 #include "Paint.h"
 #include "PaintP.h"
 #include "xpaint.h"
+#include "messages.h"
 #include "misc.h"
 #include "image.h"
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#if defined(SYSV) || defined(SVR4)
+#if defined(SYSV) || defined(SVR4) || defined(__CYGWIN__) || defined(__VMS )
 #include <dirent.h>
 #else
 #include <sys/dir.h>
@@ -49,6 +51,7 @@
 #include "protocol.h"
 
 #define MAX_PATH	1024
+#define NUM_FORMATS END_IMAGE_FORMATS-IMAGE_FORMATS
 
 #ifndef NOSTDHDRS
 #include <stdlib.h>
@@ -85,24 +88,20 @@ GetFileNameGetLastId()
  * This function is called by all image save functions.
  * If 'flag' is True, save the entire image; otherwise, save the region.
  */
-static void 
-stdSaveCommonCallback(Widget paint, char *file, Boolean flag, RWwriteFunc f)
+static void
+okSaveCallback(Widget paint, char *file, Boolean flag, RWwriteFunc f)
 {
     Pixmap pix, mask = None;
     int width, height;
     Colormap cmap;
     Image *image;
 
-    if (*file == '\0') {
-	Notice(paint, "No file name supplied");
-	return;
-    }
     StateSetBusy(True);
     if (flag) {
 	PwGetPixmap(paint, &pix, &width, &height);
     } else {
 	if (!PwRegionGet(paint, &pix, &mask)) {
-	    Notice(paint, "Unable to get region");
+	    Notice(paint, msgText[UNABLE_TO_GET_REGION]);
 	    StateSetBusy(False);
 	    return;
 	}
@@ -110,7 +109,7 @@ stdSaveCommonCallback(Widget paint, char *file, Boolean flag, RWwriteFunc f)
     XtVaGetValues(paint, XtNcolormap, &cmap, NULL);
 
     if ((image = PixmapToImage(paint, pix, cmap)) == NULL) {
-	Notice(paint, "Unable to create image for saving");
+	Notice(paint, msgText[UNABLE_TO_CREATE_IMAGE_FOR_SAVING]);
 	StateSetBusy(False);
 	return;
     }
@@ -119,7 +118,7 @@ stdSaveCommonCallback(Widget paint, char *file, Boolean flag, RWwriteFunc f)
 
     image->refCount++;
     if (f(file, image)) {
-	Notice(paint, "Error saving file:\n   %s", RWGetMsg());
+	Notice(paint, msgText[ERROR_SAVING_FILE], RWGetMsg());
     } else if (flag) {
 	PaintWidget p = (PaintWidget) paint;
 
@@ -134,6 +133,40 @@ stdSaveCommonCallback(Widget paint, char *file, Boolean flag, RWwriteFunc f)
     ImageDelete(image);
     StateSetBusy(False);
 }
+static void
+okOverwriteCallback(Widget shell, XtPointer data, XtPointer junk)
+{
+    void **ptr = data;
+    okSaveCallback(shell, (char *)ptr[0], ptr[1]?True:False, 
+                   (RWwriteFunc)ptr[2]);
+}
+static void
+nokOverwriteCallback(Widget shell, XtPointer data, XtPointer junk)
+{
+}
+
+static void 
+stdSaveCommonCallback(Widget paint, char *file, Boolean flag, RWwriteFunc f)
+{
+    char msg[512];
+    static void *ptr[3];
+    struct stat buf;
+
+    if (*file == '\0') {
+	Notice(paint, msgText[NO_FILE_NAME_SUPPLIED]);
+	return;
+    }
+    if (!stat(file, &buf)) {
+        sprintf(msg, msgText[OVERWRITE_FILE], file);
+	ptr[0] = file;
+	ptr[1] = (void *)((int)flag);
+	ptr[2] = (void *)f;
+        AlertBox(paint, msg, okOverwriteCallback, 
+                             nokOverwriteCallback, (XtPointer)ptr);
+	return;
+    }
+    okSaveCallback(paint, file, flag, f);
+}
 
 static void 
 saveRegionFileCallback(Widget paint, XtPointer str, XtPointer func)
@@ -164,7 +197,7 @@ StdSaveRegionFile(Widget w, XtPointer paintArg, XtPointer junk)
     if (PwRegionGet(paint, NULL, NULL))
 	GetFileName(paint, True, NULL, saveRegionFileCallback, NULL);
     else
-	Notice(paint, "No region selected presently");
+	Notice(paint, msgText[NO_REGION_SELECTED_PRESENTLY]);
 }
 
 void 
@@ -230,16 +263,18 @@ typedef struct arg_s {
     XtCallbackProc okFunc;
     void *type;
     Boolean isRead, isSimple;
-    Widget w, text, scrollbar;
+    Widget w;
     XtPointer closure;
     char dirname[MAX_PATH];
-    Widget list, cwd_w, info;
-    int first;
+    Widget shell, parent, browser,
+           form, title_w, name, vport, list, cwd_w, info,
+           ok, cancel;
+    Widget format[NUM_FORMATS+2];
+    int first, numf, oldwidth;
     Boolean isPoped;
     /*
     **	Use for caching, list purposes
      */
-    Widget parent;
     int browserType;
     struct arg_s *next;
 } arg_t;
@@ -270,6 +305,45 @@ emptyList(arg_t * arg)
     XtFree((XtPointer) strs);
 }
 
+static void
+browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flg)
+{
+    Dimension width, height, width2;
+    Position x, y;
+    int i, width1, height1, dw = 0;
+
+    XtVaGetValues(XtParent(l->parent), 
+		  XtNwidth, &width, XtNheight, &height, NULL);
+    if (l->oldwidth!= 0) dw = (int)width - l->oldwidth;   
+    l->oldwidth = (int)width;
+    width1 = (int)width - 120; if (width1<60) width1 = 60;
+    height1 = (int)height - 120; if (height1<60) height1 = 60;
+
+    XtResizeWidget(l->browser, width, height, 0);
+    XtResizeWidget(l->parent, width, height, 0);   
+    XtVaSetValues(l->browser, XtNwidth, width, XtNheight, height, NULL);
+    XtVaSetValues(l->parent, XtNwidth, width, XtNheight, height, NULL);   
+    XtResizeWidget(l->name, width1-2, 20, 1);
+    XtResizeWidget(l->form, width1 + 10, height, 0);
+    XtVaGetValues(l->list, XtNlongest, &width2, NULL);
+    XtVaSetValues(l->list, XtNdefaultColumns, 1, NULL);
+    XtVaSetValues(l->list, XtNwidth, 
+		  (width2>width1-30)?width2+10:width1-20, NULL);
+    XtResizeWidget(l->vport, width1, height1, 1);     
+    XtVaSetValues(l->vport, XtNwidth, width1, XtNheight, height1, NULL);
+    XtVaGetValues(l->ok, XtNx, &x, XtNy, &y, NULL);
+    if (dw) XtMoveWidget(l->ok, x+dw, y);
+    XtVaGetValues(l->cancel, XtNx, &x, XtNy, &y, NULL);  
+    if (dw) XtMoveWidget(l->cancel, x+dw, y);   
+    if (dw) for (i=0; i<l->numf; i++) {
+        XtVaGetValues(l->format[i], XtNx, &x, XtNy, &y, NULL);  
+        XtMoveWidget(l->format[i], x+dw, y);   
+    }
+    XtVaGetValues(l->cwd_w, XtNx, &x, NULL);
+    XtMoveWidget(l->cwd_w, x, height - 57);
+    XtMoveWidget(l->info, x, height - 30);
+}
+
 static char *
 doDirname(arg_t * arg, char *file)
 {
@@ -309,8 +383,10 @@ doDirname(arg_t * arg, char *file)
     } else {
 	strcpy(newPath, arg->dirname);
     }
+#ifndef __VMS
     if (strcmp(newPath, "/") != 0)
 	strcat(newPath, "/");
+#endif
     while (*file != '\0') {
 	char *ep;
 
@@ -375,7 +451,7 @@ static void
 setCWD(arg_t * arg, char *dir)
 {
     DIR *dirp;
-#if defined(SYSV) || defined(SVR4) || defined(__alpha)
+#if defined(SYSV) || defined(SVR4) || defined(__alpha) || defined(__CYGWIN__)
     struct dirent *e;
 #else
     struct direct *e;
@@ -439,7 +515,11 @@ setCWD(arg_t * arg, char *dir)
 	/*
 	**  Now only if it is a file.
 	 */
+#ifdef  __EMX__
+	{
+#else
 	if ((statbuf.st_mode & (S_IFMT & ~S_IFLNK)) == 0) {
+#endif
 	    list[i++] = XtNewString(nm);
 	    fileCount++;
 	}
@@ -457,11 +537,11 @@ setCWD(arg_t * arg, char *dir)
 	XtVaSetValues(arg->cwd_w, XtNlabel, dir, NULL);
     }
     XawListChange(arg->list, list, i, 0, True);
-    XtVaSetValues(arg->text, XtNstring, "", NULL);
-    sprintf(infoStr, "%d Directories, %d Files", dirCount, fileCount);
+    XtVaSetValues(arg->name, XtNstring, "", NULL);
+    sprintf(infoStr, "%d %s, %d %s", 
+        dirCount, msgText[DIRECTORIES], fileCount, msgText[FILES]);
     XtVaSetValues(arg->info, XtNlabel, infoStr, NULL);
-
-    if ((sb = XtNameToWidget(arg->scrollbar, "vertical")) != None) {
+    if ((sb = XtNameToWidget(arg->vport, "vertical")) != None) {
 	float top = 0.0;
 
 	XawScrollbarSetThumb(sb, top, -1.0);
@@ -481,7 +561,7 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
     char *cp;
     char *nm;
 
-    XtVaGetValues(arg->text, XtNstring, &str, NULL);
+    XtVaGetValues(arg->name, XtNstring, &str, NULL);
     if (str == NULL || *str == '\0') {
 	XawListReturnStruct *lr = XawListShowCurrent(arg->list);
 
@@ -494,20 +574,24 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
     **	  if not try and read/write the file.
      */
     if ((file = doDirname(arg, str)) == NULL) {
-	Notice(w, "No such file or directory:\n	    %s", str);
+	Notice(w, msgText[NO_SUCH_FILE_OR_DIRECTORY], str);
 	return;
     }
     if (stat(file, &statbuf) >= 0 && (statbuf.st_mode & S_IFDIR) != 0) {
 	setCWD(arg, file);
+        browserResized(arg->parent, arg, NULL, False); 
 	return;
     }
     if ((cp = strrchr(file, '/')) != NULL) {
 	*cp = '\0';
-	if (stat(file, &statbuf) >= 0 && (statbuf.st_mode & S_IFDIR) != 0)
+	if (stat(file, &statbuf) >= 0 && (statbuf.st_mode & S_IFDIR) != 0) {
 	    setCWD(arg, file);
+            browserResized(arg->parent, arg, NULL, False);
+	}
 	*cp = '/';
     }
     arg->isPoped = False;
+    arg->oldwidth = 0;
     XtPopdown(GetShell(w));
 
     if (arg->isSimple) {
@@ -532,7 +616,8 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
 	} else {
 	    Image *image = f(file);
 	    if (image == NULL)
-		Notice(w, "Unable to open input file \"%s\"\n	%s", file, RWGetMsg());
+		Notice(w, msgText[UNABLE_TO_OPEN_INPUT_FILE], file, 
+                          RWGetMsg());
 	    else
 		arg->okFunc(arg->w, (XtPointer) file, (XtPointer) image);
 	}
@@ -545,6 +630,7 @@ cancelCallback(Widget w, XtPointer argArg, XtPointer junk)
 {
     arg_t *arg = (arg_t *) argArg;
     arg->isPoped = False;
+    arg->oldwidth = 0;   
     XtPopdown(GetShell(w));
 }
 
@@ -560,14 +646,14 @@ listCallback(Widget bar, XtPointer argArg, XtPointer itemArg)
     else
 	label = item->string;
 
-    XtVaGetValues(arg->text, XtNstring, &str, NULL);
+    XtVaGetValues(arg->name, XtNstring, &str, NULL);
 
     if (strcmp(str, label) == 0)
 	okCallback(bar, argArg, NULL);
     else
-	XtVaSetValues(arg->text, XtNstring, label, NULL);
+	XtVaSetValues(arg->name, XtNstring, label, NULL);
 }
-
+   
 static Widget
 buildBrowser(Widget parent, char *title, arg_t * arg)
 {
@@ -576,6 +662,8 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
     Widget form;
     Widget title_w, name, list, vport, cwd, info;
 
+    arg->parent = parent;
+
     if (!inited) {
 	inited = True;
 
@@ -592,13 +680,14 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
     getcwd(arg->dirname, sizeof(arg->dirname));
 #endif
 
-    form = XtVaCreateManagedWidget("broswer",
+    form = XtVaCreateManagedWidget("browser",
 				   formWidgetClass, parent,
 				   XtNborderWidth, 0,
 				   XtNright, XtChainRight,
 				   XtNleft, XtChainLeft,
 				   XtNtop, XtChainTop,
 				   NULL);
+    arg->form = form;
 
     title_w = XtVaCreateManagedWidget("title",
 				      labelWidgetClass, form,
@@ -608,6 +697,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
 				      XtNbottom, XtChainTop,
 				      XtNjustify, XtJustifyLeft,
 				      NULL);
+    arg->title_w = title_w;
 
     name = XtVaCreateManagedWidget("name",
 				   asciiTextWidgetClass, form,
@@ -620,6 +710,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
 				   XtNtop, XtChainTop,
 				   XtNwidth, 250,
 				   NULL);
+    arg->name = name;
 
     vport = XtVaCreateManagedWidget("vport",
 				    viewportWidgetClass, form,
@@ -631,17 +722,22 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
 				    XtNuseBottom, True,
 				    XtNuseRight, True,
 				    XtNwidth, 250,
-				    XtNheight, 200,
+				    XtNheight, 380,
 				    NULL);
+    arg->vport = vport;
 
     list = XtVaCreateManagedWidget("files",
 				   listWidgetClass, vport,
 				   XtNborderWidth, 1,
 				   XtNverticalList, True,
+				   XtNforceBars, False,
+/*				   
 				   XtNforceColumns, True,
 				   XtNdefaultColumns, 1,
+ */
 				   XtNnumberStrings, 0,
 				   NULL);
+    arg->list = list;
 
     cwd = XtVaCreateManagedWidget("cwd",
 				  labelWidgetClass, form,
@@ -653,6 +749,8 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
 				  XtNjustify, XtJustifyLeft,
 				  XtNresizable, True,
 				  NULL);
+    arg->cwd_w = cwd;
+   
     info = XtVaCreateManagedWidget("info",
 				   labelWidgetClass, form,
 				   XtNborderWidth, 0,
@@ -661,43 +759,42 @@ buildBrowser(Widget parent, char *title, arg_t * arg)
 				   XtNbottom, XtChainBottom,
 				   XtNjustify, XtJustifyLeft,
 				   NULL);
+    arg->info = info;
 
     XtAddCallback(list, XtNcallback, listCallback, (XtPointer) arg);
-
-    arg->info = info;
-    arg->cwd_w = cwd;
-    arg->list = list;
-    arg->text = name;
-    arg->scrollbar = vport;
+    XtAddEventHandler(list, ButtonPressMask, False,
+		      (XtEventHandler) mousewheelScroll, (XtPointer) 1);
 
     arg->first = True;
 
     setCWD(arg, doDirname(arg, NULL));
-
     return form;
 }
 
 static Widget
-buildOpenBroswer(Widget w, arg_t * arg)
+buildOpenBrowser(Widget w, arg_t * arg)
 {
     Widget shell, browser, okButton, cancelButton, form;
     Widget toggle, firstToggle = None;
     XtAccelerators accel;
     XtTranslations toglt;
-    int i;
+    int i, j, k;
+    char *name_format;
     char **list;
-    Arg args[3];
+    Arg args[4];
     int nargs = 0;
     
     
     shell = XtVisCreatePopupShell("filebrowser",
 				  transientShellWidgetClass, GetToplevel(w),
 				  args, nargs);
+    arg->shell = shell;
     form = XtVaCreateManagedWidget("form",
 				   formWidgetClass, shell,
 				   XtNborderWidth, 0,
 				   NULL);
-    browser = buildBrowser(form, "Open File Named:", arg);
+    browser = buildBrowser(form, msgText[OPEN_FILE_NAMED], arg);
+    arg->browser = browser;
 
     accel = XtParseAcceleratorTable("#override\n\
 					 <Key>Return: set() notify() unset()\n\
@@ -711,6 +808,7 @@ buildOpenBroswer(Widget w, arg_t * arg)
 				       XtNleft, XtChainRight,
 				       XtNright, XtChainRight,
 				       NULL);
+    arg->ok = okButton;
     cancelButton = XtVaCreateManagedWidget("cancel",
 					   commandWidgetClass, form,
 					   XtNfromHoriz, okButton,
@@ -718,29 +816,45 @@ buildOpenBroswer(Widget w, arg_t * arg)
 					   XtNleft, XtChainRight,
 					   XtNright, XtChainRight,
 					   NULL);
+    arg->cancel = cancelButton;
 
     toggle = okButton;
     toglt = XtParseTranslationTable("<BtnDown>,<BtnUp>: set() notify()");
 
     list = RWtableGetReaderList();
 
+    j = 0;
+    k = 0;
     for (i = 0; list[i] != NULL; i++) {
+        while (strncmp(list[i], msgText[IMAGE_FORMATS+j], strlen(list[i]))) {
+	    if (j<NUM_FORMATS) ++j;
+	}
+	name_format = index(msgText[IMAGE_FORMATS+j], ' ');
+	if (name_format) 
+	   ++name_format;
+	else
+	   name_format = list[i];
+	if (j<NUM_FORMATS) ++j;
+
 	toggle = XtVaCreateManagedWidget(list[i],
 					 toggleWidgetClass, form,
 					 XtNtranslations, toglt,
 					 XtNradioGroup, firstToggle,
 					 XtNfromVert, toggle,
 					 XtNfromHoriz, browser,
+					 XtNlabel, name_format,
 					 XtNtop, XtChainBottom,
 					 XtNbottom, XtChainBottom,
 					 XtNleft, XtChainRight,
 					 XtNright, XtChainRight,
 					 NULL);
+        arg->format[k] = toggle;
+        k++;
+
 	if (firstToggle == None) {
 	    arg->type = NULL;
 	    XtVaSetValues(toggle, XtNstate, True,
-			  XtNvertDistance, 100,
-			  NULL);
+			  XtNvertDistance, 29, NULL);
 	    firstToggle = toggle;
 	}
 	XtAddCallback(toggle, XtNcallback, fileTypeCallback, (XtPointer) arg);
@@ -748,39 +862,44 @@ buildOpenBroswer(Widget w, arg_t * arg)
 
     arg->isSimple = False;
     arg->isRead = True;
+    arg->numf = k;
 
     XtAddCallback(okButton, XtNcallback, okCallback, (XtPointer) arg);
     XtAddCallback(cancelButton, XtNcallback, cancelCallback, (XtPointer) arg);
     AddDestroyCallback(shell,
 		       (DestroyCallbackFunc) cancelCallback, (XtPointer) arg);
 
-    XtSetKeyboardFocus(form, arg->text);
-    XtInstallAccelerators(arg->text, okButton);
+    XtSetKeyboardFocus(form, arg->name);
+    XtInstallAccelerators(arg->name, okButton);
 
     return shell;
 }
 
 static Widget
-buildSaveBroswer(Widget w, arg_t * arg)
+buildSaveBrowser(Widget w, arg_t * arg)
 {
     Widget shell, browser, okButton, cancelButton, form;
     Widget toggle, firstToggle = None;
     XtAccelerators accel;
+
     XtTranslations toglt;
-    int i;
+    int i, j, k;
+    char *name_format;
     char **list, *rdr = NULL;
-    Arg args[3];
+    Arg args[4];
     int nargs=0;
     
     shell = XtVisCreatePopupShell("filebrowser",
 			       transientShellWidgetClass, GetToplevel(w),
 				args, nargs);
+    arg->shell = shell;
     form = XtVaCreateManagedWidget("form",
 				   formWidgetClass, shell,
 				   XtNborderWidth, 0,
 				   NULL);
-    browser = buildBrowser(form, "Save In File:", arg);
-
+    browser = buildBrowser(form, msgText[SAVE_IN_FILE], arg);
+    arg->browser = browser;
+   
     accel = XtParseAcceleratorTable("#override\n\
 					 <Key>Return: set() notify() unset()\n\
 					 <Key>Linefeed: set() notify() unset()");
@@ -791,11 +910,13 @@ buildSaveBroswer(Widget w, arg_t * arg)
 				       XtNaccelerators, accel,
 				       XtNbottom, XtChainTop,
 				       NULL);
+    arg->ok = okButton;      
     cancelButton = XtVaCreateManagedWidget("cancel",
 					   commandWidgetClass, form,
 					   XtNfromHoriz, okButton,
 					   XtNbottom, XtChainTop,
 					   NULL);
+    arg->cancel = cancelButton;
 
     toggle = okButton;
     toglt = XtParseTranslationTable("<BtnDown>,<BtnUp>: set() notify()");
@@ -804,9 +925,23 @@ buildSaveBroswer(Widget w, arg_t * arg)
     if (RWtableGetWriter(GraphicGetReaderId(w)) != NULL) {
 	rdr = (char *) GraphicGetReaderId(w);
     }
+
+    j = 0;
+    k = 0;
     for (i = 0; list[i] != NULL; i++) {
+        while (strncmp(list[i], msgText[IMAGE_FORMATS+j], strlen(list[i]))) {
+	    if (j<NUM_FORMATS) ++j;
+	}
+	name_format = index(msgText[IMAGE_FORMATS+j], ' ');
+	if (name_format) 
+	   ++name_format;
+	else
+	   name_format = list[i];
+	if (j<NUM_FORMATS) ++j;
+
 	toggle = XtVaCreateManagedWidget(list[i],
 					 toggleWidgetClass, form,
+					 XtNlabel, name_format,
 					 XtNtranslations, toglt,
 					 XtNradioGroup, firstToggle,
 					 XtNfromVert, toggle,
@@ -814,9 +949,12 @@ buildSaveBroswer(Widget w, arg_t * arg)
 					 XtNtop, XtChainBottom,
 					 XtNbottom, XtChainBottom,
 					 NULL);
+        arg->format[k] = toggle;
+        k++;
+
 	if (firstToggle == None) {
 	    arg->type = NULL;
-	    XtVaSetValues(toggle, XtNvertDistance, 100, NULL);
+	    XtVaSetValues(toggle, XtNvertDistance, 29, NULL);
 	    firstToggle = toggle;
 	    if (rdr == NULL)
 		XtVaSetValues(toggle, XtNstate, True, NULL);
@@ -830,14 +968,15 @@ buildSaveBroswer(Widget w, arg_t * arg)
 
     arg->isSimple = False;
     arg->isRead = False;
+    arg->numf = k;
 
     XtAddCallback(okButton, XtNcallback, okCallback, (XtPointer) arg);
     XtAddCallback(cancelButton, XtNcallback, cancelCallback, (XtPointer) arg);
     AddDestroyCallback(shell,
 		       (DestroyCallbackFunc) cancelCallback, (XtPointer) arg);
 
-    XtSetKeyboardFocus(form, arg->text);
-    XtInstallAccelerators(arg->text, okButton);
+    XtSetKeyboardFocus(form, arg->name);
+    XtInstallAccelerators(arg->name, okButton);
 
     return shell;
 }
@@ -847,19 +986,21 @@ buildSimpleBrowser(Widget w, arg_t * arg, Boolean isSave)
 {
     Widget shell, browser, okButton, cancelButton, form;
     XtAccelerators accel;
-    Arg args[3];
+    Arg args[4];
     int nargs = 0;
 
     shell = XtVisCreatePopupShell("filebrowser",
 				  transientShellWidgetClass, GetToplevel(w),
 				  args, nargs);
+    arg->shell = shell;
     form = XtVaCreateManagedWidget("form",
 				   formWidgetClass, shell,
 				   XtNborderWidth, 0,
 				   NULL);
-    browser = buildBrowser(form, isSave ? "Save In File:"
-			   : "Load from File:"
-			   ,arg);
+    browser = buildBrowser(form, isSave ? 
+			   msgText[SAVE_IN_FILE] : msgText[LOAD_FROM_FILE],
+			   arg);
+    arg->browser = browser;   
 
     accel = XtParseAcceleratorTable("#override\n\
 					 <Key>Return: set() notify() unset()\n\
@@ -871,21 +1012,24 @@ buildSimpleBrowser(Widget w, arg_t * arg, Boolean isSave)
 				       XtNaccelerators, accel,
 				       XtNbottom, XtChainTop,
 				       NULL);
+    arg->ok = okButton;
     cancelButton = XtVaCreateManagedWidget("cancel",
 					   commandWidgetClass, form,
 					   XtNfromHoriz, okButton,
 					   XtNbottom, XtChainTop,
 					   NULL);
+    arg->cancel = cancelButton;   
 
     arg->isSimple = True;
+    arg->numf = 0;
 
     XtAddCallback(okButton, XtNcallback, okCallback, (XtPointer) arg);
     XtAddCallback(cancelButton, XtNcallback, cancelCallback, (XtPointer) arg);
     AddDestroyCallback(shell,
 		       (DestroyCallbackFunc) cancelCallback, (XtPointer) arg);
 
-    XtSetKeyboardFocus(form, arg->text);
-    XtInstallAccelerators(arg->text, okButton);
+    XtSetKeyboardFocus(form, arg->name);
+    XtInstallAccelerators(arg->name, okButton);
 
     return shell;
 }
@@ -909,7 +1053,7 @@ freeArg(Widget w, arg_t * arg)
 
     *pp = arg->next;
 
-    XtDestroyWidget(GetShell(arg->text));
+    XtDestroyWidget(GetShell(arg->name));
     XtFree((XtPointer) arg);
 }
 
@@ -950,10 +1094,10 @@ getArg(Widget w, Boolean type, Boolean * built)
 
     switch (type) {
     case 0:
-	shell = buildOpenBroswer(w, cur);
+	shell = buildOpenBrowser(w, cur);
 	break;
     case 1:
-	shell = buildSaveBroswer(w, cur);
+	shell = buildSaveBrowser(w, cur);
 	break;
     case 2:
 	shell = buildSimpleBrowser(w, cur, True);
@@ -980,13 +1124,15 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
     arg_t *arg;
     Boolean built;
     Position x, y;
+    Widget shell;
+    int i;
 
     XtVaGetValues(GetShell(w), XtNx, &x, XtNy, &y, NULL);
 
     arg = getArg(w, type, &built);
     arg->closure = data;
 
-    XtVaSetValues(GetShell(arg->text), XtNx, x + 24, XtNy, y + 24, NULL);
+    XtVaSetValues(GetShell(arg->name), XtNx, x + 24, XtNy, y + 24, NULL);
 
     if (def != NULL && *def != '\0') {
 	char *cp, dirname[MAX_PATH];
@@ -998,10 +1144,10 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
 	else if (cp == NULL)
 	    cp = dirname;
 
-	if (built)		/* This browser has not been used before, init it. */
+	if (built | Global.explore)		/* This browser has not been used before, init it. */
 	    setCWD(arg, doDirname(arg, dirname));
 
-	XtVaSetValues(arg->text, XtNstring, cp,
+	XtVaSetValues(arg->name, XtNstring, cp,
 		      XtNinsertPosition, strlen(cp),
 		      NULL);
     } else if (!built) {
@@ -1014,5 +1160,33 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
     arg->w = w;
 
     arg->isPoped = True;
-    XtPopup(GetShell(arg->text), XtGrabNone);
+    arg->oldwidth = 0;   
+    shell = GetShell(arg->name);
+
+    XtPopup(shell, XtGrabNone);
+    XtUnmanageChild(arg->cwd_w);
+    XtUnmanageChild(arg->title_w); 
+    XtUnmanageChild(arg->name);
+    XtUnmanageChild(arg->info);
+    XtUnmanageChild(arg->vport); 
+    XtUnmanageChild(arg->ok);    
+    XtUnmanageChild(arg->cancel); 
+    XtUnmanageChild(arg->parent);
+    XtUnmanageChild(arg->browser);
+    for (i=0; i<arg->numf; i++) XtUnmanageChild(arg->format[i]);
+
+    XMapWindow(XtDisplay(arg->parent), XtWindow(arg->parent));
+    XMapWindow(XtDisplay(arg->browser), XtWindow(arg->browser));   
+    XMapWindow(XtDisplay(arg->title_w), XtWindow(arg->title_w));
+    XMapWindow(XtDisplay(arg->info), XtWindow(arg->info));
+    XMapWindow(XtDisplay(arg->vport), XtWindow(arg->vport));
+    XMapWindow(XtDisplay(arg->cwd_w), XtWindow(arg->cwd_w));
+    XMapWindow(XtDisplay(arg->name), XtWindow(arg->name));
+    XMapWindow(XtDisplay(arg->ok), XtWindow(arg->ok));
+    XMapWindow(XtDisplay(arg->cancel), XtWindow(arg->cancel));
+    for (i=0; i<arg->numf; i++)
+        XMapWindow(XtDisplay(arg->format[i]), XtWindow(arg->format[i]));
+
+    XtAddEventHandler(XtParent(arg->parent), StructureNotifyMask, False,
+		      (XtEventHandler) browserResized, (XtPointer) arg);
 }
