@@ -12,20 +12,22 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: fontSelect.c,v 1.5 1996/10/29 08:48:44 torsten Exp $ */
+/* $Id: fontSelect.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#include <math.h>
+#include <stdio.h>
 
 #include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/Paned.h>
-#include <X11/Xaw/List.h>
-#include <X11/Xaw/AsciiText.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/Viewport.h>
 #include <X11/Xos.h>
-#include <math.h>
-#include <stdio.h>
+
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Paned.h"
+#include "xaw_incdir/List.h"
+#include "xaw_incdir/AsciiText.h"
+#include "xaw_incdir/Command.h"
+#include "xaw_incdir/Viewport.h"
 
 /*
 **  swiped from X11/Xfuncproto.h
@@ -45,12 +47,19 @@
 #endif
 
 #include "xpaint.h"
+#include "messages.h"
 #include "misc.h"
 #include "operation.h"
 #include "ops.h"
 #include "graphic.h"
 #include "protocol.h"
 
+#if defined(XAWPLAIN) || defined(XAW3D)
+#define BORDERWIDTH 1
+#else
+#define BORDERWIDTH 0
+#endif
+
 typedef struct {
     char *fontFamily;
     char *registry;
@@ -74,10 +83,12 @@ typedef struct {
     int number;
     FontName *names;
     char **namesStorage;
-    Widget text, paint;
-    Widget family, weight, point, spacing;
+    Widget shell, form1, vport, text, paint;
+    Widget spacinglabel, familylabel, weightlabel, pointlabel;
+    Widget spacing, family, weight, point;
     Widget expect, pointSelect;
     Widget fontName;
+    Widget fontnamelabel;
     Boolean expectFlag;
 } arg_t;
 
@@ -402,6 +413,7 @@ match(arg_t * arg)
 	/*
 	**  Add the family name
 	 */
+
 	nm = getShortFamily(cur->fontFamily, cur->foundry, cur->registry);
 	for (curStr = familyList; curStr != NULL; curStr = curStr->next) {
 	    if (strcmp(curStr->str, nm) == 0)
@@ -457,7 +469,7 @@ match(arg_t * arg)
 
     if (count == 1 && last != NULL) {
 	char buf[512];
-	XFontStruct *info;
+	XFontStruct *info = NULL;
 	sprintf(buf, "-%s-%s-%s-%s-%s-*-*-%s-*-*-%s-*-%s-*",
 		last->foundry,
 		last->fontFamily,
@@ -468,6 +480,8 @@ match(arg_t * arg)
 		last->spacing,
 		last->registry);
 
+        routine = "fontSelect";
+        XSetErrorHandler(privateXErrorHandler);
 	if ((info = XLoadQueryFont(XtDisplay(arg->text), buf)) != NULL) {
 	    XtVaSetValues(arg->text,
 			  XtNfont, info,
@@ -475,7 +489,8 @@ match(arg_t * arg)
 		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijlmnopqrstuvwxyz0123456789",
 			  NULL);
 	    XtVaSetValues(arg->fontName, XtNstring, buf, NULL);
-	}
+	} else
+            XtVaSetValues(arg->fontName, XtNstring, "??!!", NULL);
     } else {
 	XtVaSetValues(arg->text, XtNstring, "", NULL);
 	XtVaSetValues(arg->fontName, XtNstring, "", NULL);
@@ -642,12 +657,14 @@ closeCallback(Widget w, XtPointer argArg, XtPointer junk)
     XtPopdown(GetShell(arg->family));
 }
 
+/*
 static void 
 okCallback(Widget w, XtPointer argArg, XtPointer junk)
 {
     applyCallback(w, argArg, junk);
     closeCallback(w, argArg, junk);
 }
+*/
 
 static void 
 listCallback(Widget w, XtPointer argArg, XtPointer junk)
@@ -663,6 +680,8 @@ expectCallback(Widget w, XtPointer argArg, XEvent * event, Boolean * junk)
 {
     arg_t *arg = (arg_t *) argArg;
 
+    mousewheelScroll(w, (void *)1, event, junk);
+
     if (event->type == ButtonRelease) {
 	if (arg->expect != None)
 	    match(arg);
@@ -685,14 +704,58 @@ pointSizeAction(Widget w, XEvent * event, String * prms, Cardinal * nprms)
     match(theArg);
 }
 
+void
+fontboxResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flg)
+{
+    Dimension width, height, height1, height2;
+    XtVaGetValues(l->shell, XtNwidth, &width, XtNheight, &height, NULL);
+    XtVaGetValues(l->form1, XtNheight, &height1, NULL);
+    if (width<360) width = 360;
+    if (height1<150) height1 = 150;
+    XtResizeWidget(l->shell, width, height, 0);
+    XtResizeWidget(l->form1, width, height1, 0);
+    height1 -= 32;
+   
+    XtMoveWidget(XtParent(l->spacing),  3, 27);
+    XtResizeWidget(XtParent(l->spacing), 66, height1, BORDERWIDTH);
+    XtVaSetValues(XtParent(l->spacing), XtNwidth, 66, NULL);
+    XtMoveWidget(XtParent(l->family), 74, 27);
+    XtResizeWidget(XtParent(l->family), width-278, height1, BORDERWIDTH);
+    XtVaSetValues(XtParent(l->family), XtNwidth, width-278, NULL);   
+    XtMoveWidget(XtParent(l->weight), width-198, 27);
+    XtResizeWidget(XtParent(l->weight), 139, height1, BORDERWIDTH);
+    XtVaSetValues(XtParent(l->weight), XtNwidth, 139, NULL);   
+    XtMoveWidget(XtParent(l->point), width-54, 27);
+    XtResizeWidget(XtParent(l->point), 51, height1, BORDERWIDTH);
+    XtVaSetValues(XtParent(l->point), XtNwidth, 51, NULL);
+   
+    XtMoveWidget(l->spacinglabel, -2, 4);
+    XtMoveWidget(l->familylabel, 90, 4);
+    XtMoveWidget(l->weightlabel, width-196, 4);
+    XtMoveWidget(l->pointlabel, width-64, 4);
+
+    /* XtVaGetValues(l->text, XtNheight, &height2, NULL); */
+    XtResizeWidget(l->fontName, width-104, 22, BORDERWIDTH);
+    XtVaSetValues(l->fontName, XtNwidth, width-104, NULL);   
+#ifdef XAW3D   
+    height2 = height-height1-152;
+#else
+    height2 = height-height1-148;
+#endif
+    if (height2<4) height2 = 4;
+    XtResizeWidget(l->text, width-8, height2, BORDERWIDTH);
+    XtVaSetValues(l->text, XtNwidth, width-8, NULL);
+    XtMoveWidget(l->text, 4, 66);
+}
+
 void 
 FontSelect(Widget w, Widget paint)
 {
     static XtActionsRec pointAct =
     {"point-size-ok", (XtActionProc) pointSizeAction};
     static Widget shell = None;
-    Widget pane, form, selectLabel, form2;
-    Widget applyButton, okButton, doneButton, vport, label;
+    Widget pane, form1, form2, subform, vport, selectLabel;
+    Widget applyButton, doneButton, label;
     arg_t *arg = XtNew(arg_t);
 
     if (shell != None) {
@@ -703,45 +766,48 @@ FontSelect(Widget w, Widget paint)
     StateSetBusyWatch(True);
 
     shell = XtVaCreatePopupShell("fontSelect",
-				 topLevelShellWidgetClass, GetToplevel(w),
+				 topLevelShellWidgetClass, Global.toplevel,
 				 NULL);
+    arg->shell = shell;
 
     pane = XtVaCreateManagedWidget("pane",
-				   panedWidgetClass, shell,
+				   panedWidgetClass, arg->shell,
 				   XtNborderWidth, 0,
 				   NULL);
 
     label = XtVaCreateManagedWidget("title",
 				    labelWidgetClass, pane,
-			 XtNlabel, "Select the font properties you wish",
+	        XtNlabel, msgText[SELECT_THE_FONT_PROPERTIES_YOU_WISH],
 				    XtNborderWidth, 0,
 				    XtNshowGrip, False,
 				    NULL);
 
-    form = XtVaCreateManagedWidget("form",
+    form1 = XtVaCreateManagedWidget("form",
 				   formWidgetClass, pane,
 				   XtNborderWidth, 0,
 				   NULL);
+    arg->form1 = form1;
 
     /*
     **	lists of items to select
      */
-    label = XtVaCreateManagedWidget("spacingLabel",
-				    labelWidgetClass, form,
+    arg->spacinglabel = XtVaCreateManagedWidget("spacingLabel",
+				    labelWidgetClass, form1,
 				    XtNborderWidth, 0,
 				    XtNtop, XtChainTop,
 				    XtNbottom, XtChainTop,
 				    NULL);
     vport = XtVaCreateManagedWidget("vport",
-				    viewportWidgetClass, form,
+				    viewportWidgetClass, form1,
 				    XtNuseBottom, True,
 				    XtNuseRight, True,
-				    XtNforceBars, True,
+				    XtNforceBars, False,
 				    XtNallowHoriz, False,
 				    XtNallowVert, True,
-				    XtNfromVert, label,
+				    XtNfromVert, arg->spacinglabel,
 				    XtNtop, XtChainTop,
 				    NULL);
+    arg->vport = vport;
     arg->spacing = XtVaCreateManagedWidget("spacing",
 					   listWidgetClass, vport,
 					   XtNverticalList, True,
@@ -749,21 +815,21 @@ FontSelect(Widget w, Widget paint)
 					   XtNdefaultColumns, 1,
 					   XtNnumberStrings, 0,
 					   NULL);
-    label = XtVaCreateManagedWidget("familyLabel",
-				    labelWidgetClass, form,
+    arg->familylabel = XtVaCreateManagedWidget("familyLabel",
+				    labelWidgetClass, form1,
 				    XtNborderWidth, 0,
 				    XtNfromHoriz, vport,
 				    XtNtop, XtChainTop,
 				    XtNbottom, XtChainTop,
 				    NULL);
     vport = XtVaCreateManagedWidget("vport",
-				    viewportWidgetClass, form,
+				    viewportWidgetClass, form1,
 				    XtNuseBottom, True,
 				    XtNuseRight, True,
-				    XtNforceBars, True,
+				    XtNforceBars, False,
 				    XtNallowHoriz, False,
 				    XtNallowVert, True,
-				    XtNfromVert, label,
+				    XtNfromVert, arg->familylabel,
 				    XtNfromHoriz, vport,
 				    XtNtop, XtChainTop,
 				    NULL);
@@ -774,22 +840,22 @@ FontSelect(Widget w, Widget paint)
 					  XtNdefaultColumns, 1,
 					  XtNnumberStrings, 0,
 					  NULL);
-    label = XtVaCreateManagedWidget("weightLabel",
-				    labelWidgetClass, form,
+    arg->weightlabel = XtVaCreateManagedWidget("weightLabel",
+				    labelWidgetClass, form1,
 				    XtNborderWidth, 0,
 				    XtNfromHoriz, vport,
 				    XtNtop, XtChainTop,
 				    XtNbottom, XtChainTop,
 				    NULL);
     vport = XtVaCreateManagedWidget("vport",
-				    viewportWidgetClass, form,
+				    viewportWidgetClass, form1,
 				    XtNuseBottom, True,
 				    XtNuseRight, True,
-				    XtNforceBars, True,
+				    XtNforceBars, False,
 				    XtNallowHoriz, False,
 				    XtNallowVert, True,
 				    XtNfromHoriz, vport,
-				    XtNfromVert, label,
+				    XtNfromVert, arg->weightlabel,
 				    XtNtop, XtChainTop,
 				    NULL);
     arg->weight = XtVaCreateManagedWidget("weight",
@@ -799,22 +865,22 @@ FontSelect(Widget w, Widget paint)
 					  XtNdefaultColumns, 1,
 					  XtNnumberStrings, 0,
 					  NULL);
-    label = XtVaCreateManagedWidget("pointLabel",
-				    labelWidgetClass, form,
+    arg->pointlabel = XtVaCreateManagedWidget("pointLabel",
+				    labelWidgetClass, form1,
 				    XtNborderWidth, 0,
 				    XtNfromHoriz, vport,
 				    XtNtop, XtChainTop,
 				    XtNbottom, XtChainTop,
 				    NULL);
     vport = XtVaCreateManagedWidget("vport",
-				    viewportWidgetClass, form,
+				    viewportWidgetClass, form1,
 				    XtNuseBottom, True,
 				    XtNuseRight, True,
-				    XtNforceBars, True,
+				    XtNforceBars, False,
 				    XtNallowHoriz, False,
 				    XtNallowVert, True,
 				    XtNfromHoriz, vport,
-				    XtNfromVert, label,
+				    XtNfromVert, arg->pointlabel,
 				    XtNtop, XtChainTop,
 				    NULL);
     arg->point = XtVaCreateManagedWidget("point",
@@ -828,31 +894,31 @@ FontSelect(Widget w, Widget paint)
     /*
     **	The text area and buttons
      */
-    form = XtVaCreateManagedWidget("form2",
+    form2 = XtVaCreateManagedWidget("form2",
 				   formWidgetClass, pane,
 				   XtNborderWidth, 0,
 				   NULL);
-    form2 = XtVaCreateManagedWidget("subForm",
-				    formWidgetClass, form,
+    subform = XtVaCreateManagedWidget("subForm",
+				    formWidgetClass, form2,
 				    XtNborderWidth, 0,
 				    XtNtop, XtChainTop,
 				    XtNbottom, XtChainTop,
 				    NULL);
 
     selectLabel = XtVaCreateWidget("pointLabel",
-				   labelWidgetClass, form2,
+				   labelWidgetClass, subform,
 				   XtNborderWidth, 0,
 				   XtNleft, XtChainLeft,
 				   XtNright, XtChainLeft,
 				   NULL);
 
-    XtAppAddActions(XtWidgetToApplicationContext(form2), &pointAct, 1);
+    XtAppAddActions(XtWidgetToApplicationContext(subform), &pointAct, 1);
 
     arg->pointSelect = XtVaCreateWidget("pointSelect",
-					asciiTextWidgetClass, form2,
+					asciiTextWidgetClass, subform,
 					XtNleft, XtChainLeft,
 					XtNright, XtChainRight,
-					XtNfromHoriz, selectLabel,
+					XtNhorizDistance, 97,
 					XtNeditType, XawtextEdit,
 					XtNwrap, XawtextWrapNever,
 					XtNlength, 8,
@@ -864,18 +930,18 @@ FontSelect(Widget w, Widget paint)
 					Ctrl<Key>J: point-size-ok()\n"),
 					NULL);
 
-    label = XtVaCreateManagedWidget("fontNameLabel",
-				    labelWidgetClass, form,
+    arg->fontnamelabel = XtVaCreateManagedWidget("fontNameLabel",
+				    labelWidgetClass, form2,
 				    XtNborderWidth, 0,
-				    XtNfromVert, form2,
+				    XtNfromVert, subform,
 				    XtNleft, XtChainLeft,
 				    XtNtop, XtChainTop,
 				    XtNbottom, XtChainTop,
 				    NULL);
     arg->fontName = XtVaCreateManagedWidget("fontName",
-					    asciiTextWidgetClass, form,
-					    XtNfromVert, form2,
-					    XtNfromHoriz, label,
+					    asciiTextWidgetClass, form2,
+					    XtNfromVert, subform,
+					    XtNhorizDistance, 100,
 					    XtNallowResize, True,
 					    XtNstring, "",
 					    XtNtop, XtChainTop,
@@ -884,7 +950,7 @@ FontSelect(Widget w, Widget paint)
 					    NULL);
 
     arg->text = XtVaCreateManagedWidget("text",
-					asciiTextWidgetClass, form,
+					asciiTextWidgetClass, form2,
 					XtNfromVert, arg->fontName,
 					XtNallowResize, True,
 					XtNstring, "",
@@ -893,18 +959,19 @@ FontSelect(Widget w, Widget paint)
 					XtNdisplayCaret, False,
 					NULL);
 
+    /*
     okButton = XtVaCreateManagedWidget("ok",
-				       commandWidgetClass, form,
+				       commandWidgetClass, form2,
 				       XtNfromVert, arg->text,
 				       XtNtop, XtChainBottom,
 				       XtNbottom, XtChainBottom,
 				       XtNright, XtChainLeft,
 				       XtNleft, XtChainLeft,
 				       NULL);
+    */
     applyButton = XtVaCreateManagedWidget("apply",
-					  commandWidgetClass, form,
+					  commandWidgetClass, form2,
 					  XtNfromVert, arg->text,
-					  XtNfromHoriz, okButton,
 					  XtNtop, XtChainBottom,
 					  XtNbottom, XtChainBottom,
 					  XtNright, XtChainLeft,
@@ -912,7 +979,7 @@ FontSelect(Widget w, Widget paint)
 					  NULL);
 
     doneButton = XtVaCreateManagedWidget("done",
-					 commandWidgetClass, form,
+					 commandWidgetClass, form2,
 					 XtNfromVert, arg->text,
 					 XtNfromHoriz, applyButton,
 					 XtNtop, XtChainBottom,
@@ -926,7 +993,7 @@ FontSelect(Widget w, Widget paint)
     **	Init the world
      */
     arg->paint = paint;
-    arg->namesStorage = XListFonts(XtDisplay(shell), "*", 32767, &arg->number);
+    arg->namesStorage = XListFonts(XtDisplay(arg->shell), "*", 32767, &arg->number);
     shortFamily = (struct shortFamily_s *)
 	XtMalloc(arg->number * sizeof(struct shortFamily_s));
     shortSizes = (struct shortSizes_s *)
@@ -941,7 +1008,7 @@ FontSelect(Widget w, Widget paint)
 	list[1] = arg->pointSelect;
 	XtManageChildren(list, 2);
     } else {
-	XtVaSetValues(form2, XtNwidth, 1, XtNheight, 1, NULL);
+	XtVaSetValues(subform, XtNwidth, 1, XtNheight, 1, NULL);
     }
 
     XtAddEventHandler(arg->spacing, ButtonPressMask | ButtonReleaseMask,
@@ -958,13 +1025,47 @@ FontSelect(Widget w, Widget paint)
     XtAddCallback(arg->weight, XtNcallback, listCallback, (XtPointer) arg);
     XtAddCallback(arg->point, XtNcallback, listCallback, (XtPointer) arg);
 
+    /*
     XtAddCallback(okButton, XtNcallback, okCallback, (XtPointer) arg);
+    */
     XtAddCallback(applyButton, XtNcallback, applyCallback, (XtPointer) arg);
     XtAddCallback(doneButton, XtNcallback, closeCallback, (XtPointer) arg);
-    AddDestroyCallback(shell, (DestroyCallbackFunc) closeCallback, arg);
+    AddDestroyCallback(arg->shell, (DestroyCallbackFunc) closeCallback, arg);
 
-    XtPopup(shell, XtGrabNone);
+    XtPopup(arg->shell, XtGrabNone);
     StateSetBusyWatch(False);
 
+    XtUnmanageChild(arg->spacinglabel);
+    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->spacinglabel));
+    XtUnmanageChild(arg->familylabel);
+    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->familylabel));
+    XtUnmanageChild(arg->weightlabel);
+    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->weightlabel));
+    XtUnmanageChild(arg->pointlabel);
+    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->pointlabel));
+    XtUnmanageChild(XtParent(arg->pointSelect));
+    XMapWindow(XtDisplay(arg->shell), XtWindow(XtParent(arg->pointSelect)));
+    XtUnmanageChild(arg->fontName);
+    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->fontName));
+    XtUnmanageChild(arg->fontnamelabel);
+    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->fontnamelabel));
+    XtUnmanageChild(arg->text);
+    XMapWindow(XtDisplay(arg->shell), XtWindow(arg->text));
+    /*
+    XtUnmanageChild(XtParent(arg->spacing));
+    XMapWindow(XtDisplay(arg->shell), XtWindow(XtParent(arg->spacing)));
+    XtUnmanageChild(XtParent(arg->family));
+    XMapWindow(XtDisplay(arg->shell), XtWindow(XtParent(arg->family)));
+    XtUnmanageChild(XtParent(arg->weight));
+    XMapWindow(XtDisplay(arg->shell), XtWindow(XtParent(arg->weight)));
+    XtUnmanageChild(XtParent(arg->point));
+    XMapWindow(XtDisplay(arg->shell), XtWindow(XtParent(arg->point)));
+    */
+
+    XtAddEventHandler(arg->shell, StructureNotifyMask, False,
+		      (XtEventHandler) fontboxResized, (XtPointer) arg);
+    XtAddEventHandler(form1, StructureNotifyMask, False,
+		      (XtEventHandler) fontboxResized, (XtPointer) arg);
+
     theArg = arg;
 }
