@@ -13,24 +13,57 @@
 /* |								       | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: iprocess.c,v 1.13 1998/08/21 16:30:03 torsten Exp $ */
+/* $Id: iprocess.c,v 1.18 2005/03/20 20:15:32 demailly Exp $ */
 
-#include <X11/Intrinsic.h>
-#include <X11/StringDefs.h>
-#include <X11/Xatom.h>
+#include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
+#include <dlfcn.h>
+
+#include <X11/Intrinsic.h>
+#include <X11/Shell.h>
+#include <X11/StringDefs.h>
+#include <X11/Xatom.h>
+
+#include "xaw_incdir/AsciiText.h"
+#include "xaw_incdir/Command.h"
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/Paned.h"
+
+#include "Paint.h"
+#include "PaintP.h"
 #include "xpaint.h"
 #include "image.h"
+#include "menu.h"
+#include "messages.h"
 #include "misc.h"
 #include "protocol.h"
+#include "graphic.h"
+
+#ifdef XAW3D
+#define BORDERWIDTH 1
+#else
+#define BORDERWIDTH 0
+#endif
+
+typedef struct {
+    Widget shell, pane, name, program, cclog;
+    int mode;
+    char *scriptfile;
+} LocalInfo;
+
+#define BUFSIZE 65536
+
+static void *dl_filter = NULL;
 
 /*
 **  Some real image processing
  */
 
 extern struct imageprocessinfo ImgProcessInfo;
+extern FILE * openTemp(char **np);
+extern void removeTemp(void);
 
 #define CLAMP(low, value, high) \
 		if (value < low) value = low; else if (value > high) value = high
@@ -40,6 +73,46 @@ extern struct imageprocessinfo ImgProcessInfo;
 typedef float ConvMatrix[ConvMatrixSize * ConvMatrixSize];
 
 static int *histogram(Image * input);
+static LocalInfo *head;
+
+static PaintMenuItem fileMenu[] =
+{
+#define FILE_LOAD 0
+    MI_SIMPLE("load"),
+#define FILE_SAVE 1
+    MI_SIMPLE("save"),
+#define FILE_SAVEAS 2
+    MI_SIMPLE("saveas"),
+#define FILE_EDIT 3
+    MI_SIMPLE("editor"),
+#define FILE_CLOSE 4
+    MI_SIMPLE("close"),
+};
+static PaintMenuItem predefMenu[] =
+{
+#define PREDEF_FILTERS 0
+    MI_SIMPLE("filters"),
+#define PREDEF_IMAGES 1
+    MI_SIMPLE("images"),
+#define PREDEF_3D_CURVES 2
+    MI_SIMPLE("3d_curves"),
+#define PREDEF_3D_SURFACES 3
+    MI_SIMPLE("3d_surfaces"),
+#define PREDEF_LAYERS 4
+    MI_SIMPLE("layers"),
+#define PREDEF_PROCEDURES 5
+    MI_SIMPLE("procedures"),
+#define PREDEF_SEPARATOR 6
+    MI_SEPARATOR(),
+#define PREDEF_HELP 7
+    MI_SIMPLECB("help", HelpDialog, "script_editor"),
+};
+
+static PaintMenuBar printMenuBar[] =
+{
+    {None, "script_files", XtNumber(fileMenu), fileMenu},
+    {None, "script_predef", XtNumber(predefMenu), predefMenu},
+};
 
 static Image *
 convolve(Image * input, float *mat, int n,
@@ -138,7 +211,7 @@ normalize(Image * image)
 {
     int i, count;
     unsigned char *sp;
-    unsigned char *ip;
+    char *ip;
     int maxval = 0;
 
     if (image->cmapSize != 0) {
@@ -149,12 +222,12 @@ normalize(Image * image)
 	count = image->width * image->height * 3;
     }
 
-    for (ip = sp, i = 0; i < count; i++, ip++)
+    for (ip = (char*) sp, i = 0; i < count; i++, ip++)
 	if (*ip > maxval)
 	    maxval = *ip;
     if (maxval == 0)
 	return;
-    for (ip = sp, i = 0; i < count; i++, ip++)
+    for (ip = (char*) sp, i = 0; i < count; i++, ip++)
 	*ip = ((int) *ip * 255) / maxval;
 }
 
@@ -235,7 +308,7 @@ ImageEdge(Image * input)
 }
 
 Image *
-ImageEmbose(Image * input)
+ImageEmboss(Image * input)
 {
     static ConvMatrix mat =
     {
@@ -257,9 +330,7 @@ Image *
 ImageInvert(Image * input)
 {
     Image *output;
-    int i;
-    unsigned char *ip, *op;
-    int count;
+    unsigned char *op;
     int x,y;
     unsigned char *p;
 
@@ -918,7 +989,6 @@ HistogramImage(char *hist)
 }
 #endif
 
-#ifdef FEATURE_TILT
 /*
  * Tilt an image.
  */
@@ -985,7 +1055,6 @@ interpol(float x, float y, Image * input, unsigned char *op)
     }
 }
 
-
 Image *
 ImageTilt(Image * input)
 {
@@ -998,8 +1067,9 @@ ImageTilt(Image * input)
 
 
     X1 = width * ImgProcessInfo.tiltX1 / 100;
-    Y1 = height * ImgProcessInfo.tiltY1 / 100;
     X2 = width * ImgProcessInfo.tiltX2 / 100;
+
+    Y1 = height * ImgProcessInfo.tiltY1 / 100;
     Y2 = height * ImgProcessInfo.tiltY2 / 100;
 
     if (((Y1 >= 0) && (Y1 < height)) || ((X2 >= 0) && (X2 < width)))
@@ -1042,7 +1112,6 @@ ImageTilt(Image * input)
 
     return output;
 }
-#endif
 
 /*
  * Produce the 'solarization' effect seen when exposing a 
@@ -1217,7 +1286,7 @@ ImageQuantize(Image * input)
 
     clutx = 0;
 
-    /* Count all occurrances of each color */
+    /* Count all occurrences of each color */
     for (y = 0; y < h; ++y)
 	for (x = 0; x < w; ++x) {
 	    p = ImagePixel(input, x, y);
@@ -1389,6 +1458,753 @@ ImageGrey(Image * input)
     return output;
 }
 
+/*
+ * Modify RGB values of an image.
+ */
+Image *
+ImageModifyRGB(Image * input)
+{
+    Image *output = ImageNew(input->width, input->height);
+    unsigned char *p, *op = output->data;
+    int x, y;
+    double fr, fg, fb, frp, fgp, fbp;
+
+    if (!ImgProcessInfo.redshift &&
+        !ImgProcessInfo.greenshift &&
+        !ImgProcessInfo.greenshift) return NULL;
+  
+    fr = 2.0*ImgProcessInfo.redshift / (100.01-ImgProcessInfo.redshift);
+    fg = 2.0*ImgProcessInfo.greenshift / (100.01-ImgProcessInfo.greenshift);
+    fb = 2.0*ImgProcessInfo.blueshift / (100.01-ImgProcessInfo.blueshift);
+    frp = fr / 255.0;
+    fgp = fg / 255.0;
+    fbp = fb / 255.0;
+    fr = fr + 1.0;
+    fg = fg + 1.0;
+    fb = fb + 1.0;
+
+    for (y = 0; y < input->height; y++) {
+	for (x = 0; x < input->width; x++) {
+	    p = ImagePixel(input, x, y);
+	    *op++ = (char) ( 0.25 + p[0] * fr / (1.0 + p[0] * frp) );
+	    *op++ = (char) ( 0.25 + p[1] * fg / (1.0 + p[1] * fgp) );
+	    *op++ = (char) ( 0.25 + p[2] * fb / (1.0 + p[2] * fbp) );
+	}
+    }
+
+    return output;
+}
+
+/*
+ * Pipe command routines
+ */
+
+void pipe_command(command, input, output, error)
+    char *command;
+    int input(char*, int);
+    void output(const char*, int);
+    void error(const char*, int);
+{
+    int in_pipe[2];
+    int out_pipe[2];
+    int err_pipe[2];
+    int pid, bytes;
+    char buffer[BUFSIZE];
+    
+    if (pipe(in_pipe)==-1)
+    {
+        return;
+    }
+    if (pipe(out_pipe)==-1)
+    {
+        return;
+    }
+    if (pipe(err_pipe)==-1)
+    {
+        return;
+    }
+    
+    pid = fork();
+    if (pid==-1)
+    {
+        close(in_pipe[0]);
+        close(in_pipe[1]);
+        close(out_pipe[0]);
+        close(out_pipe[1]);
+        close(err_pipe[0]);
+        close(err_pipe[1]);
+        return;
+    }
+    
+    if (!pid)
+    {    /* child's thread */
+        close(in_pipe[1]); /* there's no writing on input pipe! */
+        close(out_pipe[0]); /* there's no reading of output pipe! */
+        close(err_pipe[0]); /* there's no reading of error pipe! */        
+        /* redirect stdin */
+        if (input)
+        {            
+            if (dup2(in_pipe[0], fileno(stdin))==-1)
+            {
+                exit(1);
+            }
+        }
+        /* redirect stdout */
+        if (output)
+        {            
+            if (dup2(out_pipe[1], fileno(stdout))==-1)
+            {
+                exit(1);
+            }
+        }
+        /* redirect stderr */
+        if (error)
+        {
+            if (dup2(err_pipe[1], fileno(stderr))==-1)
+            {
+                exit(1);
+            }
+        }
+                  system(command);
+        exit(1);        
+    }
+    else
+    {    /* parent's thread */
+        close(in_pipe[0]); /* there's no reading of input pipe! */
+        close(out_pipe[1]); /* there's no writing on output pipe! */
+        close(err_pipe[1]); /* there's no writing on error pipe! */
+        /* start writing to pipe */
+        if (input)
+        {            
+            while((bytes=input(buffer, BUFSIZE-1))>0)
+            {  
+                bytes = write(in_pipe[1], buffer, bytes);
+                if (bytes==-1)
+                {                  
+                    break;
+                }            
+            }
+            close(in_pipe[1]);
+        }
+        /* start reading pipe output */
+        if (output)
+        {
+            while((bytes=read(out_pipe[0], buffer, BUFSIZE-1))>0)
+                output(buffer, bytes);
+            close(out_pipe[0]);
+        }
+        /* start reading pipe error*/
+        if (error)
+        {
+            while((bytes=read(err_pipe[0], buffer, BUFSIZE-1))>0)
+                error(buffer, bytes);
+            close(err_pipe[0]);
+        }
+    }        
+}
+
+static void append_status(char *string)
+{
+char *ptr, *catptr;
+
+    XtVaGetValues(head->cclog, XtNstring, &ptr, NULL);
+    catptr = malloc(strlen(ptr)+strlen(string)+10);
+    strcpy(catptr, ptr);
+    strcat(catptr, "\n");
+    strcat(catptr, string);
+    XtVaSetValues(head->cclog, XtNstring, catptr, NULL);
+}
+
+static void CollectOutput(const char* data, int i)
+{
+    static int FlagNewLine = 1;
+    int pos=0;
+    char superstring[2000];
+
+    while(i--)
+    {
+    if(data[pos] == '\n')
+    {
+        char tmp[2000];
+        strncpy(tmp, data, pos);
+        tmp[pos] = '\0';
+        tmp[pos+1] = '\0';
+        if (!FlagNewLine)
+        {
+        strcat(superstring, tmp);
+        }
+        else
+        {
+            superstring[0] = '\0';
+        strcpy(superstring, tmp);
+        }
+        data = data + pos + 1;
+        pos = 0;
+        FlagNewLine = 1;
+            append_status(superstring);
+    }
+    else
+        pos++;
+    }
+    if (pos)
+    {
+    if (!FlagNewLine)
+        strcat(superstring, data);
+    else
+        superstring[0] = '\0';
+    FlagNewLine = 0;
+    }
+}
+
+
+/*
+ * User defined script procedures
+ */
+
+static void
+closeCallback(Widget w, XtPointer wlArg, XtPointer junk)
+{
+   LocalInfo *info = (LocalInfo *) wlArg;
+   XtDestroyWidget(info->shell);
+}
+
+static void
+compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
+{
+   LocalInfo *info = (LocalInfo *) wlArg;
+   FILE *fd;
+   void (* proc)(Widget wid);
+   Image * (* iproc)();
+   Image * image;
+   Screen *screen;
+   Visual *visual;
+   Pixmap pix;
+   Colormap cmap;
+   char *tmp, *ptr, *text;
+   char cmd[512];
+   char header[30];
+   char * error;
+   void *dl_handle = NULL;
+   int i;
+
+   /* create temporary name */
+   fd= openTemp(&tmp);
+   fclose(fd);
+
+   /* copy buffer to temporary C file */
+   XtVaGetValues(info->program, XtNstring, &text, NULL);
+   if (!text || !*text) {
+      return;
+   }
+   sprintf(cmd, "%s.c", tmp);
+   fd = fopen(cmd, "w");
+   if (!fd) return;
+
+   ptr = text;
+   i = 0;
+   while (*ptr) {
+      fputc(*ptr, fd);
+      if (i <= 25) {
+         header[i] = *ptr;
+         if (*ptr == '\n') header[i] = '\0';
+	 i++;
+      }
+      ++ptr;
+   }
+   fclose(fd);
+   header[i] = '\0';
+
+   info->mode = PREDEF_IMAGES;
+   if (strncasecmp(header, "/* Xpaint-image */", 18) == 0)
+      info->mode = PREDEF_IMAGES;
+   if (strncasecmp(header, "/* Xpaint-filter */", 19) == 0)
+      info->mode = PREDEF_FILTERS;
+   if (strncasecmp(header, "/* Xpaint-procedure */", 22) == 0)
+      info->mode = PREDEF_PROCEDURES;
+
+
+   /* free any previous dll */
+   if (info->mode == PREDEF_FILTERS && dl_filter) {
+      dlclose(dl_filter);
+      dl_filter = NULL;
+   }
+
+   /* compile C script */
+   sprintf(cmd, "cd /tmp\ngcc -I%s -c %s.c -o %s.o\n"
+                "gcc -shared -Wl,-soname,%s.so %s.o -o %s.so\n",
+	        SHAREDIR"/include",
+                tmp, tmp, strstr(tmp, "XPaint"), tmp, tmp);
+   XtVaSetValues(info->cclog, XtNstring, cmd, NULL);
+   pipe_command(cmd, NULL, CollectOutput, CollectOutput);
+
+   /* clean *.c and *.o file */
+   sprintf(cmd, "%s.c", tmp);
+   unlink(cmd);
+   sprintf(cmd, "%s.o", tmp);
+   unlink(cmd);
+
+   /* load dll library */
+   sprintf(cmd, "%s.so", tmp);
+   if (info->mode == PREDEF_FILTERS)
+      dl_handle = dl_filter = dlopen(cmd, RTLD_LAZY);
+   else
+      dl_handle = dlopen(cmd, RTLD_LAZY);
+
+   /* clean dll .so file */
+   unlink(cmd);
+   removeTemp();
+
+   if (dl_handle)
+      sprintf(cmd, "echo \"%s\"",
+	      msgText[C_SCRIPT_SUCCESSFULLY_COMPILED_AND_LINKED]);
+   else
+      sprintf(cmd, "echo \"\"; echo \"%s\"; echo \"%s\"",
+              msgText[C_SCRIPT_COULD_NOT_BE_COMPILED_OR_LINKED], dlerror());
+   pipe_command(cmd, NULL, CollectOutput, CollectOutput);
+
+   if (info->mode == PREDEF_IMAGES) {
+      if (!dl_handle) return;
+      iproc = dlsym(dl_handle, "ImageCreate");
+      if ((error = dlerror()) != NULL || !iproc) {
+	 if (!error) error = msgText[UNABLE_TO_CREATE_IMAGE];
+         Notice(Global.toplevel, error);
+         return;
+      }
+      pix = None;
+      image = iproc();
+      dlclose(dl_handle);
+      dl_handle = NULL;
+      if (image) {
+	 screen = XtScreen(Global.toplevel);
+	 visual = DefaultVisualOfScreen(screen);
+	 cmap = XCreateColormap(XtDisplay(Global.toplevel), 
+		    RootWindowOfScreen(screen), visual, AllocNone);
+	 ImageToPixmap(image, Global.toplevel, &pix, &cmap);
+      }
+      if (!pix) {
+         Notice(Global.toplevel, msgText[UNABLE_TO_CREATE_IMAGE]);
+	 return;
+      }
+      graphicCreate(makeGraphicShell(Global.toplevel), 0, 0, -1, pix, cmap);
+      return;
+   }
+
+   if (info->mode == PREDEF_PROCEDURES) {
+      if (!dl_handle) return;
+      proc = dlsym(dl_handle, "PaintProcedure");
+      if (proc) proc(Global.toplevel);
+      dlclose(dl_handle);
+      dl_handle = NULL;
+      return;
+   }
+}
+
+static void
+editCallback(Widget w, XtPointer wlArg, XtPointer junk)
+{
+   FILE *fd;
+   LocalInfo *info = (LocalInfo *) wlArg;
+   int l;
+   char *tmp, *ptr, *text;
+   char name[256];
+   char buf[2048];
+
+   /* write buffer to file */
+   fd = openTemp(&tmp);
+   fclose(fd);
+   sprintf(name, "%s.c", tmp);
+   removeTemp();
+   fd = fopen(name, "w");
+   if (!fd) {
+      Notice(w, msgText[UNABLE_TO_OPEN_FILE], name);
+      return;
+   }
+   XtVaGetValues(info->program, XtNstring, &text, NULL);
+   if (text && *text) {
+      ptr = text;
+      while (*ptr) {
+         fputc(*ptr, fd);
+         ++ptr;
+      }
+   }
+   fclose(fd);
+
+   /* call external editor */
+   sprintf(buf, "%s %s", EDITOR, name);
+   system(buf);
+
+   /* Now copy edited file into buffer */
+   fd = fopen(name, "r");
+   if (!fd) {
+      Notice(w, msgText[UNABLE_TO_OPEN_FILE], name);
+      return;
+   }
+
+   text = malloc(2);
+   *text = '\0';
+   l = 0;
+   while (fgets(buf, 2040, fd)) {
+      l += strlen(buf);
+      text = realloc(text, l+2);
+      strcat(text, buf);
+   }
+   fclose(fd);
+   unlink(name);
+   XtVaSetValues(info->program, XtNstring, text, NULL);
+}
+
+static void 
+loadFileCallbackOK(Widget w, XtPointer wlArg, char *file)
+{
+   FILE *fd;
+   LocalInfo *info = (LocalInfo *) wlArg;
+   static char *text;
+   char buf[2048];
+   int l;
+
+   if (!file || !*file) return;
+   fd = fopen(file, "r");
+   if (!fd) {
+      Notice(w, msgText[UNABLE_TO_OPEN_FILE], file);
+      return;
+   }
+   info->scriptfile = file;
+   XtVaSetValues(info->name, XtNlabel, file, NULL);
+   text = malloc(2);
+   *text = '\0';
+   l = 0;
+   while (fgets(buf, 2040, fd)) {
+      l += strlen(buf);
+      text = realloc(text, l+2);
+      strcat(text, buf);
+   }
+   fclose(fd);
+   XtVaSetValues(info->program, XtNstring, text, NULL);
+}
+
+static void
+loadFileCallback(Widget w, XtPointer wlArg, XtPointer junk)
+{
+   char buf[256];
+   *buf = '\0';
+   if (getcwd(buf, 256)) strcat(buf, "/");
+   Global.explore = True;
+   GetFileName(GetShell(w), 3, 
+               buf, (XtCallbackProc) loadFileCallbackOK, wlArg);
+   Global.explore = False;
+}
+
+static void 
+saveFileCallbackOK(Widget w, XtPointer wlArg, char *file)
+{
+   FILE *fd;
+   LocalInfo *info = (LocalInfo *) wlArg;
+   char *ptr, *text;
+ 
+   if (!file || !*file) return;
+   fd = fopen(file, "w");
+   if (!fd) {
+      Notice(w, msgText[UNABLE_TO_OPEN_FILE], file);
+      return;
+   }
+   info->scriptfile = file;
+   XtVaSetValues(info->name, XtNlabel, file, NULL);
+   XtVaGetValues(info->program, XtNstring, &text, NULL);
+   if (!text || !*text) {
+      fclose(fd);
+      return;
+   }
+   ptr = text;
+   while (*ptr) {
+      fputc(*ptr, fd);
+      ++ptr;
+   }
+   fclose(fd);
+}
+
+static void
+saveFileCallback(Widget w, XtPointer wlArg, XtPointer junk)
+{
+   LocalInfo *info = (LocalInfo *) wlArg;
+   char buf[256];
+   *buf = '\0';
+   if (getcwd(buf, 256)) strcat(buf, "/");
+   Global.explore = True;
+   if (!info->scriptfile || !info->scriptfile[0])
+      GetFileName(GetShell(w), 2, 
+               buf, (XtCallbackProc) saveFileCallbackOK, wlArg);
+   else
+      saveFileCallbackOK(w, wlArg, info->scriptfile);
+   Global.explore = False;
+}
+
+static void
+saveasFileCallback(Widget w, XtPointer wlArg, XtPointer junk)
+{
+   char buf[256];
+   *buf = '\0';
+   if (getcwd(buf, 256)) strcat(buf, "/");
+   Global.explore = True;
+   GetFileName(GetShell(w), 2, 
+               buf, (XtCallbackProc) saveFileCallbackOK, wlArg);
+   Global.explore = False;
+}
+
+static void
+loadScriptCallback(Widget w, XtPointer wlArg, XtPointer junk)
+{
+   LocalInfo *info = (LocalInfo *) wlArg;
+   char directory[256];
+   int i;
+
+   info->mode = PREDEF_IMAGES;
+   for (i=0; i<PREDEF_SEPARATOR; i++)
+      if (w == predefMenu[i].widget) {
+	 info->mode = i;
+	 break;
+      }
+
+   Global.explore = True;
+   sprintf(directory, "%s/%s/", SHAREDIR"/c_scripts", 
+	              predefMenu[info->mode].name);
+   GetFileName(GetShell(w), 3, directory, 
+      (XtCallbackProc) loadFileCallbackOK, wlArg);
+   Global.explore = False;
+}
+
+void
+editorResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
+{
+    Dimension width, height;
+    Dimension width1, height1, height2;
+
+    XtVaGetValues(l->shell, XtNwidth, &width, XtNheight, &height, NULL);
+    XtResizeWidget(XtParent(l->name), width, 38, BORDERWIDTH);
+    XtResizeWidget(l->pane, width, height, BORDERWIDTH);
+    XtVaSetValues(l->pane, XtNwidth, width, XtNheight, height, NULL);
+    XtVaGetValues(XtParent(l->program), XtNheight, &height1, NULL);
+
+#ifdef XAW3DG   
+    width1 = (width>12)? width-10 : 2;
+    height1 = (height1>10)? height1-8 : 2;
+    height2 = (height>height1+63)? height - height1 - 53 : 10;
+#else
+    width1 = (width>10)? width-8 : 2;
+    height1 = (height1>10)? height1-8 : 2;
+    height2 = (height>height1+60)? height - height1 - 50 : 10;
+#endif   
+    XtResizeWidget(XtParent(l->program), width1+8, height1+8, BORDERWIDTH);
+    XtResizeWidget(l->program, width1, height1, BORDERWIDTH);
+    XtVaSetValues(XtParent(l->program), 
+       XtNwidth, width1+8, XtNheight, height1+8, NULL);
+    XtVaSetValues(l->program, 
+       XtNwidth, width1, XtNheight, height1, NULL);
+    XtMoveWidget(XtParent(l->program), 0, 39);
+    XtMoveWidget(l->program, 4, 0);
+
+    XtResizeWidget(XtParent(l->cclog), width1+8, height2+8, BORDERWIDTH);
+    XtResizeWidget(l->cclog, width1, height2, BORDERWIDTH);
+    XtVaSetValues(XtParent(l->cclog), 
+       XtNwidth, width1+8, XtNheight, height2+8, NULL);
+    XtVaSetValues(l->cclog, 
+       XtNwidth, width1, XtNheight, height2, NULL);
+    XtMoveWidget(XtParent(l->cclog), 0, 46+height1);
+    XtMoveWidget(l->cclog, 4, 0);
+}
+
+void
+ScriptEditor(Widget w, Widget paint)
+{
+    Display *dpy;
+    Window root;
+    Widget topform, midform, botform;
+    Widget bar, cancel, compile;
+    LocalInfo *info = XtNew(LocalInfo);
+    Arg args[4];
+    int nargs = 0;
+    int i;
+
+    head = info;
+    info->mode = 0;
+    info->scriptfile = NULL;
+
+    dpy = XtDisplay(GetShell(w));
+    root = RootWindowOfScreen(XtScreen(GetShell(w)));
+
+    info->shell = XtVisCreatePopupShell("XPaint / C scripts", 
+                                  topLevelShellWidgetClass,
+				  Global.toplevel, args, nargs);
+    info->pane = XtVaCreateManagedWidget("pane", panedWidgetClass, info->shell,
+                                   NULL);
+    topform = XtVaCreateManagedWidget("topform", formWidgetClass, info->pane, 
+                                      NULL);
+    bar = MenuBarCreate(topform, XtNumber(printMenuBar), printMenuBar);
+    XtVaSetValues(bar, XtNvertDistance, 2, NULL);
+    
+    info->name = XtVaCreateManagedWidget("name",
+				    labelWidgetClass, topform,
+				    XtNlabel, "",
+				    XtNwidth, 350,
+				    XtNfromHoriz, bar,
+                                    XtNhorizDistance, 15,
+                                    XtNvertDistance, 6,
+				    XtNborderWidth, 0,
+                                    NULL);
+
+    compile = XtVaCreateManagedWidget("compile",
+		 		    commandWidgetClass, topform,
+				    XtNfromHoriz, bar,
+				    XtNhorizDistance, 400,
+				    XtNvertDistance, 5,
+				    NULL);
+    cancel = XtVaCreateManagedWidget("close",
+		 		    commandWidgetClass, topform,
+				    XtNfromHoriz, compile,
+				    XtNhorizDistance, 4,
+				    XtNvertDistance, 5,
+				    NULL);
+
+    midform = XtVaCreateManagedWidget("midform", 
+                                    formWidgetClass, info->pane, NULL);
+    botform = XtVaCreateManagedWidget("botform", 
+                                    formWidgetClass, info->pane, NULL);
+
+    info->program = 
+          XtVaCreateManagedWidget("program", asciiTextWidgetClass, midform,
+                                  XtNeditType, XawtextEdit,
+                                  XtNresizable, True,
+                                  XtNwidth, 620,
+                                  XtNheight, 300,
+				  XtNstring, NULL,
+				  XtNscrollHorizontal, XawtextScrollWhenNeeded,
+				  XtNscrollVertical, XawtextScrollWhenNeeded,
+				  NULL);
+
+    info->cclog = 
+          XtVaCreateManagedWidget("cclog", asciiTextWidgetClass, botform,
+                                  XtNeditType, XawtextRead,
+                                  XtNresizable, True,
+                                  XtNwidth, 620,
+                                  XtNheight, 100,
+				  XtNstring, NULL,
+				  XtNscrollHorizontal, XawtextScrollWhenNeeded,
+				  XtNscrollVertical, XawtextScrollWhenNeeded,
+				  NULL);
+
+    AddDestroyCallback(info->shell,
+                       (DestroyCallbackFunc) closeCallback, (XtPointer) info);
+
+    XtAddCallback(fileMenu[FILE_LOAD].widget, XtNcallback, 
+		     (XtCallbackProc) loadFileCallback, (XtPointer) info);
+    XtAddCallback(fileMenu[FILE_SAVE].widget, XtNcallback, 
+		     (XtCallbackProc) saveFileCallback, (XtPointer) info);
+    XtAddCallback(fileMenu[FILE_SAVEAS].widget, XtNcallback, 
+		     (XtCallbackProc) saveasFileCallback, (XtPointer) info);
+    XtAddCallback(fileMenu[FILE_EDIT].widget, XtNcallback, 
+		     (XtCallbackProc) editCallback, (XtPointer) info);
+    XtAddCallback(fileMenu[FILE_CLOSE].widget, XtNcallback, 
+		     (XtCallbackProc) closeCallback, (XtPointer) info);
+    XtAddCallback(compile, XtNcallback,
+                  (XtCallbackProc)compileCallback, (XtPointer) info);
+    XtAddCallback(cancel, XtNcallback,
+                  (XtCallbackProc)closeCallback, (XtPointer) info);
+
+    for (i=0; i<PREDEF_SEPARATOR; i++)
+       XtAddCallback(predefMenu[i].widget, XtNcallback, 
+		     (XtCallbackProc) loadScriptCallback, (XtPointer) info);
+
+    /* XtRealizeWidget(info->shell); */
+    XtPopup(info->shell, XtGrabNone);
+    XtVaSetValues(topform, XtNshowGrip, False, NULL);
+    XtVaSetValues(midform, XtNshowGrip, True, NULL);
+    XtVaSetValues(botform, XtNshowGrip, True, NULL);
+
+    XtUnmanageChild(info->name);
+    XMapWindow(dpy, XtWindow(info->name));
+    XtUnmanageChild(cancel);
+    XMapWindow(dpy, XtWindow(cancel));
+    XtUnmanageChild(compile);
+    XMapWindow(dpy, XtWindow(compile));
+    XtUnmanageChild(info->program);
+    XMapWindow(dpy, XtWindow(info->program));
+    XtUnmanageChild(info->cclog);
+    XMapWindow(dpy, XtWindow(info->cclog));      
+   
+    XtAddEventHandler(midform, StructureNotifyMask, False,
+		      (XtEventHandler) editorResized, (XtPointer) info);
+    XtAddEventHandler(info->shell, StructureNotifyMask, False,
+		      (XtEventHandler) editorResized, (XtPointer) info);
+    XtAddEventHandler(info->program, ButtonPressMask, False,
+		      (XtEventHandler) mousewheelScroll, (XtPointer) NULL);
+    XtAddEventHandler(info->cclog, ButtonPressMask, False,
+		      (XtEventHandler) mousewheelScroll, (XtPointer) NULL);
+}
+
+/* convenience routines */
+int
+RegionX()
+{
+    PaintWidget pw = (PaintWidget)Global.curpaint;
+    return pw->paint.region.rect.x;
+}
+int
+RegionY()
+{
+    PaintWidget pw = (PaintWidget)Global.curpaint;
+    return pw->paint.region.rect.y;
+}
+int
+RegionWidth()
+{
+    PaintWidget pw = (PaintWidget)Global.curpaint;
+    return pw->paint.region.rect.width;
+}
+int
+RegionHeight()
+{
+    PaintWidget pw = (PaintWidget)Global.curpaint;
+    return pw->paint.region.rect.height;
+}
+
+Image *
+CanvasImage()
+{
+    PaintWidget pw = (PaintWidget)Global.curpaint;
+    Image * image;
+
+    image = PixmapToImage((Widget)pw, GET_PIXMAP(pw), Global.region.cmap);
+    return image;
+}
+
+Boolean
+isFilterDefined()
+{
+    return (dl_filter)? True : False;
+}
+
+Image *
+ImageUserDefined(Image * input)
+{
+    Image *output;
+    char * error;
+    void (* proc)(Image *, Image *);
+    
+    if (!dl_filter) return NULL;
+
+    proc = dlsym(dl_filter, "FilterProcess");
+    if ((error = dlerror()) != NULL) {
+       Notice(Global.toplevel, error);
+       return input;
+    }
+
+    output = ImageNew(input->width, input->height);
+
+    proc(input, output);
+
+    return output;
+}
+
+
 #define GRAY(p)		(((p)[0]*169 + (p)[1]*256 + (p)[2]*87)/512)
 #define SQ(x)		((x)*(x))
 
