@@ -13,11 +13,17 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: main.c,v 1.16 1997/09/03 19:31:07 torsten Exp $ */
+/* $Id: main.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#ifdef __VMS
+#define XtDisplay XTDISPLAY
+#define XtScreen XTSCREEN
+#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/Shell.h>
 #include <X11/StringDefs.h>
+#include <ctype.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <time.h>
@@ -34,11 +40,21 @@
 #include "misc.h"
 #include "operation.h"
 #include "graphic.h"
+#include "messages.h"
 #include "protocol.h"
+#include "image.h"
 #include "rw/rwTable.h"
 
 #include "XPaintIcon.xpm"
 
+String *helpText;
+int helpNum;
+String *msgText;
+int msgNum;
+
+char lang[10];
+char *routine;
+
 extern void BrushInit(Widget toplevel);
 
 static char *appDefaults[] =
@@ -49,10 +65,20 @@ static char *appDefaults[] =
 
 typedef struct {
     String visualType;
+    String lang;
     String size;
+    String winsize;
     String help;
     String rcFile;
+    String helpFile;
+    String msgFile;
+    int zoom;
+    Boolean fullpopup;
+    Boolean nomenubar;
     Boolean popped;
+    Boolean snapshot;
+    Boolean clipboard;
+    Boolean horizontal;
     Boolean nowarn;
 } AppInfo;
 
@@ -62,14 +88,34 @@ static XtResource resources[] =
 {
     {"visualType", "VisualType", XtRString, sizeof(String),
    XtOffset(AppInfo *, visualType), XtRImmediate, (XtPointer) "default"},
+    {"lang", "Lang", XtRString, sizeof(String),
+     XtOffset(AppInfo *, lang), XtRImmediate, (XtPointer) NULL},
     {"size", "Size", XtRString, sizeof(String),
      XtOffset(AppInfo *, size), XtRImmediate, (XtPointer) "640x480"},
+    {"winsize", "WinSize", XtRString, sizeof(String),
+     XtOffset(AppInfo *, winsize), XtRImmediate, (XtPointer) "0x0"},
     {"rcFile", "RcFile", XtRString, sizeof(String),
      XtOffset(AppInfo *, rcFile), XtRImmediate, (XtPointer) NULL},
     {"help", "Help", XtRString, sizeof(String),
      XtOffset(AppInfo *, help), XtRImmediate, (XtPointer) NULL},
+    {"helpFile", "HelpFile", XtRString, sizeof(String),
+     XtOffset(AppInfo *, helpFile), XtRImmediate, (XtPointer) NULL},
+    {"msgFile", "MsgFile", XtRString, sizeof(String),
+     XtOffset(AppInfo *, msgFile), XtRImmediate, (XtPointer) NULL},
+    {"zoom", "Zoom", XtRInt, sizeof(int),
+     XtOffset(AppInfo *, zoom), XtRImmediate, (XtPointer) 1},
+    {"fullpopup", "FullPopup", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, fullpopup), XtRImmediate, (XtPointer) True},
+    {"nomenubar", "NoMenubar", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, nomenubar), XtRImmediate, (XtPointer) False},
     {"popped", "Popped", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, popped), XtRImmediate, (XtPointer) False},
+    {"snapshot", "Snapshot", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, snapshot), XtRImmediate, (XtPointer) False},
+    {"clipboard", "Clipboard", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, clipboard), XtRImmediate, (XtPointer) False},
+    {"horizontal", "Horizontal", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, horizontal), XtRImmediate, (XtPointer) False},
     {"nowarn", "NoWarn", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, nowarn), XtRImmediate, (XtPointer) False},
 };
@@ -78,9 +124,20 @@ static XrmOptionDescRec options[] =
 {
     {"-24", ".visualType", XrmoptionNoArg, (XtPointer) "true24"},
     {"-12", ".visualType", XrmoptionNoArg, (XtPointer) "cmap12"},
+    {"-language", ".lang", XrmoptionSepArg, (XtPointer) NULL},
     {"-size", ".size", XrmoptionSepArg, (XtPointer) NULL},
+    {"-winsize", ".winsize", XrmoptionSepArg, (XtPointer) "0x0"},
     {"-rcFile", ".rcFile", XrmoptionSepArg, (XtPointer) NULL},
+    {"-helpFile", ".helpFile", XrmoptionSepArg, (XtPointer) NULL},
+    {"-msgFile", ".msgFile", XrmoptionSepArg, (XtPointer) NULL},
+    {"-zoom", ".zoom", XrmoptionSepArg, (XtPointer) "-1"},
+    {"-simplepopup", ".fullpopup", XrmoptionNoArg, (XtPointer) "False"},
+    {"-fullpopup", ".fullpopup", XrmoptionNoArg, (XtPointer) "True"},
+    {"-nomenubar", ".nomenubar", XrmoptionNoArg, (XtPointer) "True"},
     {"-popped", ".popped", XrmoptionNoArg, (XtPointer) "True"},
+    {"-snapshot", ".snapshot", XrmoptionNoArg, (XtPointer) "True"},
+    {"-clipboard", ".clipboard", XrmoptionNoArg, (XtPointer) "True"},
+    {"-horizontal", ".horizontal", XrmoptionNoArg, (XtPointer) "True"},
     {"-nowarn", ".nowarn", XrmoptionNoArg, (XtPointer) "True"},
     {"-help", ".help", XrmoptionNoArg, (XtPointer) "command"},
     {"+help", ".help", XrmoptionSepArg, (XtPointer) NULL},
@@ -93,6 +150,19 @@ static XrmOptionDescRec options[] =
 **  Public query functions for application defaults
  */
 
+void 
+SetDefaultWHZ(int w, int h, int zoom)
+{
+    char str[40];
+    if (w<=0) w = 1;
+    if (h<=0) h = 1;
+    if (zoom<=0) zoom = -1;
+    if (zoom>32) zoom = 32;
+    sprintf(str, "%dx%d", w, h);
+    appInfo.size = strdup(str);
+    appInfo.zoom = zoom;
+}
+
 void 
 GetDefaultWH(int *w, int *h)
 {
@@ -104,6 +174,55 @@ GetDefaultWH(int *w, int *h)
     *h = height;
 }
 
+void 
+GetPaintWH(int *w, int *h)
+{
+    int x, y;
+    unsigned int width, height;
+
+    XParseGeometry(appInfo.winsize, &x, &y, &width, &height);
+    *w = width;
+    *h = height;
+}
+
+void 
+GetInitZoom(int *z)
+{
+    *z = appInfo.zoom;
+    if (*z<=0) *z = -1;
+    if (*z>32) *z = 32;
+}
+
+void 
+SetMenuBarHidden(Boolean flag)
+{
+    appInfo.nomenubar = flag;
+}
+
+Boolean
+ThereIsNoMenuBar()
+{
+    return appInfo.nomenubar;
+}
+
+void 
+SetFullMenuPopup(Boolean flag)
+{
+    appInfo.fullpopup = flag;
+}
+
+Boolean
+IsFullPopup()
+{
+    return appInfo.fullpopup;
+}
+
+Boolean
+ToolsAreHorizontal()
+{
+    return appInfo.horizontal;
+}
+
 char *
 GetDefaultRC(void)
 {
@@ -136,7 +255,7 @@ SetIconImage(Widget w)
  	myattributes.closeness = 65535;
  	if (XpmCreatePixmapFromData(dpy, iconWindow,
 				    XPaintIcon_xpm, &icon, NULL, &myattributes)) {
-	    fprintf(stderr, "Cannot install XPM icon\n");
+	    fprintf(stderr, msgText[CANNOT_INSTALL_XPM_ICON]);
 	    exit(1);
  	}
 	GetPixmapWHD(dpy, icon, &iconW, &iconH, NULL);
@@ -156,7 +275,8 @@ usage(char *prog, char *msg)
 {
     if (msg)
 	fprintf(stderr, "%s\n", msg);
-    fprintf(stderr, "Usage for %s (XPaint %s):\n", prog, XPAINT_VERSION);
+    fprintf(stderr, "%s %s (XPaint %s):\n", 
+            msgText[USAGE_FOR], prog, XPAINT_VERSION);
     HelpTextOutput(stderr, appInfo.help == NULL ? "command" : appInfo.help);
     exit(1);
 }
@@ -185,7 +305,7 @@ workProc(LocalInfo * l)
     if ((v = ReadMagic(file)) != NULL)
 	GraphicOpenFile(l->toplevel, file, v);
     else
-	Notice(l->toplevel, "Unable to open input file \"%s\"\n	%s",
+	Notice(l->toplevel, msgText[UNABLE_TO_OPEN_INPUT_FILE],
 	       file, RWGetMsg());
 
     StateSetBusy(False);
@@ -212,11 +332,6 @@ GetFutureVisual(Widget *toplevel, int desiredDepth, int desiredVisual,
 		int desiredVisualID, Boolean *valid)
 {
     Display *display = XtDisplay(*toplevel);
-    Colormap colormap; 
-    Window window; 
-    XSetWindowAttributes attributes; 
-    unsigned long valuemask; 
-    int screen_num = 0;
     int i, visnum = 0;
 	
     XVisualInfo rtnval;
@@ -262,7 +377,7 @@ GetFutureVisual(Widget *toplevel, int desiredDepth, int desiredVisual,
     }
 	
     if (visualsMatched == 0) {
-        fprintf(stderr, "No match found for visual\n");
+        fprintf(stderr, msgText[NO_MATCH_FOUND_FOR_VISUAL]);
 	exit(1);
     }
 	
@@ -274,8 +389,145 @@ GetFutureVisual(Widget *toplevel, int desiredDepth, int desiredVisual,
     return rtnval;
 }
 
+void
+LoadMessages()
+{
+    FILE *fd;
+    int i, j;
+    char buf[256];
+    static char alert[] = "!! No messages found !!";
+    static char failure[] = "Memory allocation failure\n";
 
-void 
+    helpNum = 0; 
+    fd = fopen(appInfo.helpFile, "r");
+    if (!fd) {
+        sprintf(buf, "%s/help/Help", SHAREDIR);
+	fd = fopen(buf, "r");
+    }
+    if (fd) {
+        while (!feof(fd)) {
+            ++helpNum;
+            helpText = (String *)realloc(helpText, helpNum*sizeof(String));
+            fgets(buf, 250, fd);
+            i = strlen(buf)-1;
+            if (i>0 && buf[i] == '\n') buf[i] = '\0';
+            helpText[helpNum-1] = strdup(buf);
+        }
+        fclose(fd);
+    } else {
+        fprintf(stderr, "No help file found !!\n");
+    }
+
+    msgNum = 0; 
+    fd = fopen(appInfo.msgFile, "r");
+    if (!fd) {
+        sprintf(buf, "%s/messages/Messages", SHAREDIR);
+	fd = fopen(buf, "r");
+    }
+    if (fd) {
+        while (!feof(fd)) {
+            fgets(buf, 250, fd);
+	    if (*buf == '#') {
+	         if (!strncmp(buf+1, "END_MESSAGES", 12)) break;
+	         continue;
+	    }
+            ++msgNum;
+            msgText = (String *)realloc(msgText, msgNum*sizeof(String));
+	    if (!msgText) {
+	        fprintf(stderr, failure);
+	        exit(1);
+	    }
+            i = 0;
+            j = 0;
+            while (buf[i]) {
+	        if (buf[i] == '\n') break;
+	        if (buf[i] == '\\' && buf[i+1] == 'n') {
+                     ++i;
+		     buf[j] = '\n';
+		} else
+		if (j<i)
+		     buf[j] = buf[i];
+                ++i;
+                ++j;
+	    }
+	    buf[j] = '\0';
+            msgText[msgNum-1] = strdup(buf);
+	    if (!msgText[msgNum-1]) {
+	        fprintf(stderr, failure);
+	        exit(1);
+	    }
+        }
+        fclose(fd);
+    } else {
+        msgNum = END_MESSAGES;
+        msgText = (String *)realloc(msgText, msgNum*sizeof(String));
+        for (i=0; i<msgNum; i++) msgText[i] = alert;
+	fprintf(stderr, "%s\n\n", msgText[0]);
+    }
+}
+
+String GetAppDefaultFile()
+{
+    char *lg;
+    char name[256];
+    int i;
+    FILE *fd = NULL;
+
+    if (!*lang && (lg = getenv("LANG"))) strncpy(lang, lg, 8);
+    if (*lang) {
+        for (i=0; i<=8 ; i++) lang[i] = tolower(lang[i]);
+    }
+    else
+        strcpy(lang, "en");
+    sprintf(name, "%s/XPaint_%s", XAPPLOADDIR, lang);
+    fd = fopen(name, "r");
+    if (*lang && !fd) {
+        lang[2] = '\0';
+        sprintf(name, "%s/XPaint_%s", XAPPLOADDIR, lang);
+	fd = fopen(name, "r");
+    }   
+    if (fd) {
+        fclose(fd);
+	sprintf(name, "XPaint_%s", lang);
+	return strdup(name);
+    } else
+        return strdup("XPaint");
+}
+
+
+static void
+checkRCsize()
+{
+    FILE *fd;
+    char buf[256];
+    char *ptr;
+    int w, h, i;
+    if (!strcmp(appInfo.size, "640x480")) {
+        if (appInfo.rcFile)
+	    strcpy(buf, appInfo.rcFile);
+	else
+	    strcpy(buf, ".XPaintrc");
+	if ((fd = fopen(buf, "r"))) {
+	    i = 0;
+            while (!feof(fd)) {
+                fgets(buf, 250, fd);
+		if (*buf && isspace(*buf)) ++i;
+		if (!strncasecmp(buf, "defaultsize", 11)) {
+		    ptr = buf + 12;
+		    while (isspace(*ptr)) ++ptr;
+	            if (sscanf(ptr, "%dx%d", &w, &h) == 2) {
+	                 SetDefaultWHZ(w, h, appInfo.zoom);
+		    }
+		    break;
+		}
+		if (i >= 4) break;
+	    }
+	    fclose(fd);
+	}
+    }
+}
+
+int
 main(int argc, char *argv[])
 {
     Display *dpy;
@@ -295,33 +547,53 @@ main(int argc, char *argv[])
     Boolean valid;
     int depth;
     int desiredDepth;
-    Colormap colormap;
     int tempargc;
-    char tempargvMem[5][1024];
-    char *tempargv[5];
-    XtAppContext app_context;
-    Display *display;
+    char **tempargv;
     char phrase[128];
+    String app_def_file;
     int desiredVisualID = -1;	 
 
     InitTypeConverters();
 	 
+    msgText = NULL;
+    helpText = NULL;
+    tempargc = argc;
+    tempargv = xmalloc(argc * sizeof(char *));
+    
+    *lang = '\0';
+    for (i = 0; i < argc; i++) {
+        tempargv[i] = xmalloc(strlen(argv[i])+1);
+	if (i < argc-1 && !strncasecmp(argv[i], "-language", strlen(argv[i])))
+	   strncpy(lang, argv[i+1], 8);
+	strcpy(tempargv[i], argv[i]);
+    }
+
+    app_def_file = GetAppDefaultFile();
+    if (!app_def_file) {
+        fprintf(stderr, "%s : %s\n",
+            msgText[CANT_FIND_APP_DEFAULTS_FILE], app_def_file);
+        exit(1);
+    }
+
     /*
      * Create a temp widget that we can interrogate.
      * This widget will not be realized. 
      * There should be a better way to do this...
      */
-    tempargc = argc;
-    for (i = 0; i < tempargc; i++) {
-        tempargv[i] = tempargvMem[i];
-	strcpy(tempargv[i], argv[i]);
-    }
-    temp = XtAppInitialize(&Global.appContext, "XPaint",
+    temp = XtAppInitialize(&Global.appContext, app_def_file,
 			   options, XtNumber(options), &tempargc, tempargv,
 			   appDefaults, args, nargs);
     XtGetApplicationResources(temp, (XtPointer) & appInfo,
 			      resources, XtNumber(resources), NULL, 0);
+
+   /*
+    **  Load help and messages files
+     */
+    LoadMessages();
+
     rdb = XtDatabase(XtDisplay(temp));
+    for (i = 0; i < tempargc; i++)
+        free(tempargv[i]);
 	 
     if (appInfo.help != NULL)
 	usage(argv[0], NULL);
@@ -380,7 +652,7 @@ main(int argc, char *argv[])
  	    desiredVisualID = atoi(vp);
 	    *vp = '\0';
 	} else {
-	    fprintf(stderr, "%s: bad visual type specification '%s'\n",
+	    fprintf(stderr, msgText[BAD_VISUAL_TYPE_SPECIFICATION],
 		    argv[0], vp);
 	    exit(1);
 	}
@@ -409,7 +681,7 @@ main(int argc, char *argv[])
 	    Global.vis = vis;
 	}
 	else {
-	    fprintf(stderr, "No match found for visual\n");
+	    fprintf(stderr, msgText[NO_MATCH_FOUND_FOR_VISUAL]);
 	    exit(1);
 	}
     }
@@ -425,32 +697,46 @@ main(int argc, char *argv[])
 
     /* destroy our temporary widget */
     XtDestroyWidget(temp);
-	 
+
     /*
     **  Create the application context
      */
 
-    toplevel = XtAppInitialize(&Global.appContext, "XPaint",
+    toplevel = XtAppInitialize(&Global.appContext, app_def_file,
 			       options, XtNumber(options), &argc, argv,
 			       appDefaults, args, nargs);
 
     XtGetApplicationResources(toplevel, (XtPointer) & appInfo,
 			      resources, XtNumber(resources), NULL, 0);
 
-	 
-	 
     if (argc != 1 && argv[1][0] == '-')
 	usage(argv[0], "Invalid option");
-	 
+
+    checkRCsize();
+
     /*
     **  A little initialization
      */
+    Global.toplevel = toplevel;
+    Global.canvas = None;
+    Global.back = None;
+    Global.patternshell = None;
+    Global.patterninfo = NULL;
     Global.region.image = NULL;
     Global.region.cmap = None;
     Global.region.width = 0;
     Global.region.height = 0;
     Global.region.pix = None;
     Global.region.mask = None;
+    Global.nbrushes = 0;
+    Global.brushes = NULL;
+    Global.join = JoinMiter;
+    Global.dashlist[0] = '\0';
+    Global.dashnumber = 0;
+    Global.dashoffset = 0;
+    Global.numregions = 0;
+    Global.transparent = 0;
+    Global.regiondata = NULL;
 
     /*
     **
@@ -463,18 +749,18 @@ main(int argc, char *argv[])
     /*
     **  GRR 960525:  check depth and warn user (use AlertBox() instead?)
      */
-    if (!appInfo.nowarn) {
+    if (!appInfo.nowarn && !appInfo.snapshot) {
 	int depth = Global.vis.depth;
 
 	/* XtVaGetValues(toplevel, XtNdepth, &depth, NULL); */
 	HelpTextOutput(stderr, "data_loss");
-	fprintf(stderr, "\nYour canvas depth is %d bits.\n\n", depth);
+	fprintf(stderr, msgText[YOUR_CANVAS_DEPTH_IS_THAT_MANY_BITS], depth);
 	switch (depth) {
 	    case 8:
 	    case 12:
-		fprintf(stderr, "     =============================================================\n");
-		fprintf(stderr, "     WARNING!  Most true-color images will suffer major data loss!\n");
-		fprintf(stderr, "     =============================================================\n");
+		fprintf(stderr, msgText[SEPARATOR_STRIP_ONE]);
+		fprintf(stderr, msgText[WARNING_ONE]);
+		fprintf(stderr, msgText[SEPARATOR_STRIP_ONE]);
 		fprintf(stderr, "\n");
 		break;
 	    case 1:
@@ -483,17 +769,17 @@ main(int argc, char *argv[])
 	    case 6:
 	    case 15:
 	    case 16:
-		fprintf(stderr, "     ================================================================\n");
-		fprintf(stderr, "     WARNING!  Most color-mapped (palette) and true-color images will\n");
-		fprintf(stderr, "     lose information irreversibly!\n");
-		fprintf(stderr, "     ================================================================\n");
+		fprintf(stderr, msgText[SEPARATOR_STRIP_TWO]);
+		fprintf(stderr, msgText[WARNING_TWO_A]);
+		fprintf(stderr, msgText[WARNING_TWO_B]);
+		fprintf(stderr, msgText[SEPARATOR_STRIP_TWO]);
 		fprintf(stderr, "\n");
 		break;
 	    case 24:
 	    case 32:
-		fprintf(stderr, "     Normal images (color-mapped, 8-bit grayscale, 24-bit true-color)\n");
-		fprintf(stderr, "     will not lose any color info, but non-true-color images may be\n");
-		fprintf(stderr, "     promoted to 24-bit mode.\n\n");
+		fprintf(stderr, msgText[WARNING_THREE_A]);
+		fprintf(stderr, msgText[WARNING_THREE_B]);
+		fprintf(stderr, msgText[WARNING_THREE_C]);
 		break;
 	    default:
 		break;
@@ -504,6 +790,7 @@ main(int argc, char *argv[])
     **  Now build and construct the widgets
      */
     Global.timeToDie = False;
+    Global.explore = False;
     app = XtWidgetToApplicationContext(toplevel);
     dpy = Global.display = XtDisplay(toplevel);
 
@@ -530,30 +817,47 @@ main(int argc, char *argv[])
     XtVaSetValues(toplevel, XtNtitle, xpTitle, NULL);
  /* XtVaSetValues(toplevel, XtNiconName, xpTitle, XtNtitle, xpTitle, NULL); */
 
+    if (appInfo.snapshot) {
+        SnapshotImage(toplevel, NULL, 0);
+    }
     /*
-    **  Realize it  (doesn't hurt)
+    **  Realize it (doesn't hurt)
      */
     XtRealizeWidget(toplevel);
 
-
     /*
     **  Now open any file on the command line
      */
     if (argc != 1) {
-	LocalInfo *l = XtNew(LocalInfo);
-	l->pos = 0;
-	l->toplevel = toplevel;
-	l->nfiles = argc - 1;
-	l->files = (char **) XtCalloc(argc, sizeof(char *));
-	for (i = 1; i < argc; i++)
-	    l->files[i - 1] = argv[i];
-	l->id = XtAppAddWorkProc(app, (XtWorkProc) workProc, (XtPointer) l);
+        if (appInfo.clipboard) {
+	   Image * image;
+	   for (i = 1; i < argc; i++) {
+  	       image = ImageFromFile(argv[i]);
+               if (image)
+                  ImageToMemory(image);
+	   }
+	} else {
+  	   LocalInfo *l = XtNew(LocalInfo);
+	   l->pos = 0;
+	   l->toplevel = toplevel;
+  	   l->nfiles = argc - 1;
+	   l->files = (char **) XtCalloc(argc, sizeof(char *));
+           for (i = 1; i < argc; i++)
+	       l->files[i - 1] = argv[i];
+	   l->id = XtAppAddWorkProc(app, (XtWorkProc) workProc, (XtPointer) l);
+	}
     } else if (appInfo.popped) {
 	/*
 	**  If nothing is coming up, bring up a blank canvas
 	 */
 	GraphicCreate(toplevel, 0);
     }
+
+    XtUnmanageChild(Global.bar);
+    XMapWindow(dpy, XtWindow(Global.bar));
+    XtUnmanageChild(Global.back);
+    XMapWindow(dpy, XtWindow(Global.back));
+
     /*
     **  MainLoop
      */
@@ -564,4 +868,6 @@ main(int argc, char *argv[])
 	XtDispatchEvent(&event);
     }
     while (!Global.timeToDie);
+    return 0;
 }
+
