@@ -2,11 +2,12 @@
  * Miscellaneous definitions and prototypes, including those from misc.c.
  */
 
-/* $Id: misc.h,v 1.26 2000/10/02 20:56:30 torsten Exp $ */
+/* $Id: misc.h,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 #if defined(HAVE_PARAM_H)
 #include <sys/param.h>
 #endif
+#include <stdio.h>
 
 /*
 **  By default everything uses drand48(),
@@ -23,7 +24,7 @@
 long random(void);
 #endif
 
-#if !defined(linux) & !defined(__EMX__) & !defined(__FreeBSD__) & !defined(__CYGWIN__)
+#if !defined(__VMS) & !defined(linux) & !defined(__EMX__) & !defined(__FreeBSD__) & !defined(__CYGWIN__)
 #if defined(BSD4_4) || defined(HPArchitecture) || defined(SGIArchitecture) || defined(_AIX) || defined(_SCO_DS)
 void srandom(unsigned int);
 #else
@@ -56,6 +57,9 @@ extern long lrand48();
 #define strcasecmp stricmp
 #endif
 
+#define ICONWIDTH 48
+#define ICONHEIGHT 44
+
 /* dialog.c */
 void AlertBox(Widget parent, char *msg, XtCallbackProc okProc,
 	      XtCallbackProc nokProc, void *data);
@@ -72,7 +76,6 @@ void FatbitsEdit(Widget paint);
 
 /* fileName.c */
 void *GetFileNameGetLastId(void);
-void StdCloneRegionFile(Widget w, XtPointer paintArg, XtPointer junk);
 void StdSaveRegionFile(Widget w, XtPointer paintArg, XtPointer junk);
 void StdSaveAsFile(Widget w, XtPointer paintArg, XtPointer junk);
 void StdSaveFile(Widget w, XtPointer paintArg, XtPointer junk);
@@ -84,6 +87,10 @@ void GetFileName(Widget w, int type, char *def,
 /* fontSelect.c */
 void FontSelect(Widget w, Widget paint);
 
+/* iprocess.c */
+Boolean isFilterDefined();
+void ScriptEditor(Widget w, Widget paint);
+
 /* grab.c */
 #ifdef __IMAGE_H__
 Image *DoGrabImage(Widget w, int width, int height);
@@ -94,27 +101,38 @@ XColor *DoGrabColor(Widget w);
 /* help.c */
 char *matchGet(char *line, char *pat);
 void HelpDialog(Widget parent, String name);
-#ifdef _STDIO_H
-void HelpTextOutput(FILE * fd, String name);
+#if defined( _STDIO_H ) || defined( __VMS ) 
+void HelpTextOutput(FILE* fd, String name);
 #endif
 void HelpInit(Widget top);
 
 /* main.c */
+void SetDefaultWHZ(int w, int h, int zoom);
 void GetDefaultWH(int *w, int *h);
+void GetPaintWH(int *w, int *h);
+void GetInitZoom(int *z);
 char *GetDefaultRC(void);
 void SetIconImage(Widget w);
+void SetMenuBarHidden(Boolean flag);
+void SetFullMenuPopup(Boolean flag);
+Boolean ThereIsNoMenuBar();
+Boolean IsFullPopup();
+Boolean ToolsAreHorizontal();
 
 /* misc.c */
+int privateXErrorHandler(Display *dpy, XErrorEvent *myerr);
 Widget GetToplevel(Widget w);
 Widget GetShell(Widget w);
 void SetIBeamCursor(Widget w);
 void SetCrossHairCursor(Widget w);
 void SetPencilCursor(Widget w);
+void SetCapAndJoin(Widget w, GC gc, int join);
 void EnlargePixmap(Display * dpy, Pixmap cursor, int zoom,
 		   Pixmap * data, Pixmap * mask);
 XRectangle *RectUnion(XRectangle * a, XRectangle * b);
 XRectangle *RectIntersect(XRectangle * a, XRectangle * b);
 void GetPixmapWHD(Display * dpy, Drawable d, int *wth, int *hth, int *dth);
+Pixmap dupPixmap(Display * dpy, Pixmap pix);
 Pixmap GetBackgroundPixmap(Widget w);
 GC GetGCX(Widget w);
 void StrToArgv(char *str, int *argc, char **argv);
@@ -126,11 +144,51 @@ void *xmalloc(size_t n);
 void AutoCrop(Widget paint);
 Widget XtVisCreatePopupShell(String name,  WidgetClass widget_class,  Widget parent, ArgList args, Cardinal num_args);
 
+/* graphic.c */
+void loadClipboard(Widget w, XtPointer junk, XtPointer junk2);
+void lineWidth(Widget w, int width);
+void changeDashStyleAction(Widget w, XEvent * event);
+void setToolIconPixmap(Widget paint, void *ptr);
+void setBrushIconPixmap(Widget paint, void *ptr);
+void setFontIcon(Widget paint);
+void setCanvasColorsIcon(Widget paint);
+void changeBackground(Widget w, XtPointer paintArg, XtPointer junk2);
+void AddItemToCanvasPalette(Widget paint, Pixel p, Pixmap pix);
+Boolean inCanvasPixmaps(Widget paint, Pixmap pix);
+
 /* pattern.c */
-void PatternEdit(Widget w, Pixmap pix, Widget button);
+void PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
+                 int npixels, int npatterns, int nbrushes);
+void checkPatternLink(Widget w, int mode);
+void setPatternLineWidth(void * ptr, int width);
+
+/* print.c */
+void PrintPopup(Widget w, XtPointer paintArg);
+void ExternPopup(Widget w, XtPointer paintArg);
+void checkExternalLink(Widget pw);
 
 /* size.c */
 void SizeSelect(Widget w, Widget paint, void (*func) (Widget, int, int, int));
 
 /* typeConvert.c */
 void InitTypeConverters(void);
+
+/* operation.c */
+void takeSnapshot(Widget w, XtPointer junk, XtPointer junk2);
+void exitPaint(Widget w, XtPointer junk, XtPointer junk2);
+void setToolIconOnWidget(Widget w);
+
+/* snapshot.c */
+void SnapshotImage(Widget w, XtPointer paintArg, int flag);
+
+
+/* brushOp.c */
+void BrushSelect(Widget w);
+void setBrushIconOnWidget(Widget w);
+
+/* rw/readWritePNM.c */
+#ifdef __IMAGE_H__
+extern int WritePNM(char *file, Image * image);
+#endif
+
+void mousewheelScroll(Widget w, void * l, XEvent * event, Boolean * flg);
