@@ -13,26 +13,30 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: operation.c,v 1.18 1997/08/13 19:45:28 torsten Exp $ */
+/* $Id: operation.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
-#include <X11/Intrinsic.h>
-#include <X11/StringDefs.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/SmeBSB.h>
-#include <X11/Xaw/Box.h>
-#include <X11/Xaw/Toggle.h>
-#include <X11/Xaw/Viewport.h>
-#include <X11/cursorfont.h>
-#include <xpm.h>
 #include <stdio.h>
 #ifndef NOSTDHDRS
 #include <stdlib.h>
 #include <unistd.h>
 #endif
 
+#include <X11/Intrinsic.h>
+#include <X11/StringDefs.h>
+#include <X11/cursorfont.h>
+
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/SmeBSB.h"
+#include "xaw_incdir/Box.h"
+#include "xaw_incdir/Toggle.h"
+#include "xaw_incdir/Viewport.h"
+
+#include <xpm.h>
+
 #include "xpaint.h"
 #include "misc.h"
 #include "menu.h"
+#include "messages.h"
 #include "Paint.h"
 #include "text.h"
 #include "ops.h"
@@ -40,47 +44,49 @@
 #include "image.h"
 #include "operation.h"
 #include "protocol.h"
-#include "cutCopyPaste.h"
+#include "region.h"
 
 /* Pixmaps for toolbox icons */
-#include "brushOp.xpm"
-#include "eraseOp.xpm"
-#include "sprayOp.xpm"
-#include "pencilOp.xpm"
-#include "dotPenOp.xpm"
-#include "dynPenOp.xpm"
-#include "lineOp.xpm"
-#include "arcOp.xpm"
-#include "fillOp.xpm"
-#include "tfillOp.xpm"
+
+#include "bitmaps/tools/brushOp.xpm"
+#include "bitmaps/tools/eraseOp.xpm"
+#include "bitmaps/tools/sprayOp.xpm"
+#include "bitmaps/tools/pencilOp.xpm"
+#include "bitmaps/tools/dotPenOp.xpm"
+#include "bitmaps/tools/dynPenOp.xpm"
+#include "bitmaps/tools/lineOp.xpm"
+#include "bitmaps/tools/arcOp.xpm"
+#include "bitmaps/tools/fillOp.xpm"
+#include "bitmaps/tools/tfillOp.xpm"
 #ifdef FEATURE_FRACTAL
-#include "ffillOp.xpm"
+#include "bitmaps/tools/ffillOp.xpm"
 #endif
-#include "smearOp.xpm"
-#include "textOp.xpm"
-#include "selectOp.xpm"
-#include "boxOp.xpm"
-#include "rayOp.xpm"
-#include "fboxOp.xpm"
-#include "ovalOp.xpm"
-#include "fovalOp.xpm"
-#include "lassoOp.xpm"
-#include "clineOp.xpm"
-#include "polyOp.xpm"
-#include "fpolyOp.xpm"
-#include "freehandOp.xpm"
-#include "ffreehandOp.xpm"
-#include "selpolyOp.xpm"
+#include "bitmaps/tools/smearOp.xpm"
+#include "bitmaps/tools/textOp.xpm"
+#include "bitmaps/tools/selectOp.xpm"
+#include "bitmaps/tools/boxOp.xpm"
+/* #include "bitmaps/tools/rayOp.xpm" */
+#include "bitmaps/tools/fboxOp.xpm"
+#include "bitmaps/tools/ovalOp.xpm"
+#include "bitmaps/tools/fovalOp.xpm"
+#include "bitmaps/tools/lassoOp.xpm"
+#include "bitmaps/tools/clineOp.xpm"
+#include "bitmaps/tools/splineOp.xpm"
+#include "bitmaps/tools/fsplineOp.xpm"
+#include "bitmaps/tools/polyOp.xpm"
+#include "bitmaps/tools/fpolyOp.xpm"
+#include "bitmaps/tools/freehandOp.xpm"
+#include "bitmaps/tools/ffreehandOp.xpm"
+#include "bitmaps/tools/selpolyOp.xpm"
 
-
-
-static void changeLineAction(Widget, XEvent *);
+/* static void changeLineAction(Widget, XEvent *); */
+static void changeFontAction(Widget, XEvent *);
 static void changeDynPencilAction(Widget, XEvent *);
 static void changeSprayAction(Widget, XEvent *);
 static void changeTFillAction(Widget, XEvent *);
 static void changeBrushAction(Widget, XEvent *);
 static void changeBrushParmAction(Widget, XEvent *);
-static void changeFontAction(Widget, XEvent *);
+static void changeFractalParmAction(Widget, XEvent *);
 
 static void 
 setOperation(Widget w, XtPointer opArg, XtPointer junk2)
@@ -98,70 +104,7 @@ setOperation(Widget w, XtPointer opArg, XtPointer junk2)
     GraphicSetOp((OperationRemoveProc) stop, (OperationAddProc) start);
 
     CurrentOp = op;
-}
-
-/*
-**  The set line width callback pair
- */
-static char currentLineWidth[10] = "1";
-
-static void 
-setWidth(Widget w, void *width)
-{
-    XtVaSetValues(w, XtNlineWidth, (int) width, NULL);
-}
-static void 
-okLineCallback(Widget w, XtPointer junk, XtPointer infoArg)
-{
-    Arg arg;
-    TextPromptInfo *info = (TextPromptInfo *) infoArg;
-    int width = atoi(info->prompts[0].rstr);
-
-    if (width < 1 || width > 1000) {
-	Notice(w, "Invalid width, must be greater than zero and less than 1000");
-	return;
-    }
-    sprintf(currentLineWidth, "%d", width);
-
-    MenuCheckItem(w, True);
-
-    if (width == 1)
-	width = 0;
-    GraphicAll(setWidth, (void *) width);
-    XtSetArg(arg, XtNlineWidth, width);
-    OperationAddArg(arg);
-}
-
-static void 
-lineWidth(Widget w, int width)
-{
-    String lbl;
-
-    XtVaGetValues(w, XtNlabel, &lbl, NULL);
-    width = atoi(lbl);
-    if (width <= 0) {
-	static TextPromptInfo info;
-	static struct textPromptInfo value[1];
-
-	value[0].prompt = "Width:";
-	value[0].str = currentLineWidth;
-	value[0].len = 4;
-	info.prompts = value;
-	info.title = "Enter desired line width";
-	info.nprompt = 1;
-
-	TextPrompt(w, "linewidth", &info, okLineCallback, NULL, NULL);
-    } else {
-	Arg arg;
-
-	if (width == 1)
-	    width = 0;
-	MenuCheckItem(w, True);
-	GraphicAll(setWidth, (void *) width);
-
-	XtSetArg(arg, XtNlineWidth, width);
-	OperationAddArg(arg);
-    }
+    GraphicAll(setToolIconPixmap, NULL);
 }
 
 /*
@@ -172,10 +115,10 @@ static void
 brushOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
 {
     TextPromptInfo *info = (TextPromptInfo *) infoArg;
-    int opacity = atoi(info->prompts[0].rstr);
+    int opacity = atof(info->prompts[0].rstr);
 
     if (opacity < 0 || opacity > 100) {
-	Notice(w, "Opacity should be between 0 and 100 %");
+	Notice(w, msgText[OPACITY_SHOULD_BE_BETWEEN_ZERO_AND_HUNDRED_PERCENT]);
 	return;
     }
     BrushSetParameters(opacity / 100.0);
@@ -189,16 +132,51 @@ brushMenuCallback(Widget w)
     static TextPromptInfo info;
     static struct textPromptInfo value[1];
 
-    value[0].prompt = "Opacity (%):";
+    value[0].prompt = msgText[OPACITY];
     value[0].str = brushOpacityStr;
     value[0].len = 4;
     info.prompts = value;
-    info.title = "Enter the desired brush parameters";
+    info.title = msgText[ENTER_THE_DESIRED_BRUSH_PARAMETERS];
     info.nprompt = 1;
 
     TextPrompt(w, "brushparams", &info, brushOkCallback, NULL, NULL);
 }
 
+/*
+** Set fractal parameter callbacks
+ */
+char fractalDensityStr[10] = "40";
+static void 
+fractalOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
+{
+    TextPromptInfo *info = (TextPromptInfo *) infoArg;
+    int opacity = atof(info->prompts[0].rstr);
+
+    if (opacity < 0 || opacity > 100) {
+	Notice(w, msgText[OPACITY_SHOULD_BE_BETWEEN_ZERO_AND_HUNDRED_PERCENT]);
+	return;
+    }
+    BrushSetParameters(opacity / 100.0);
+
+    sprintf(fractalDensityStr, "%d", opacity);
+}
+
+static void 
+fractalMenuCallback(Widget w)
+{
+    static TextPromptInfo info;
+    static struct textPromptInfo value[1];
+
+    value[0].prompt = msgText[DENSITY];
+    value[0].str = fractalDensityStr;
+    value[0].len = 4;
+    info.prompts = value;
+    info.title = msgText[ENTER_THE_DESIRED_FRACTAL_DENSITY];
+    info.nprompt = 1;
+
+    TextPrompt(w, "fractalparams", &info, fractalOkCallback, NULL, NULL);
+}
+
 /*
 ** Set dynamic pencil parameters callbacks
 */
@@ -214,17 +192,17 @@ static void dynPencilOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
     float drag = atof(info->prompts[2].rstr);
     
     if (width < 1 || width > 100) {
-	Notice(w, "Width should be between 1 and 100");
+	Notice(w, msgText[WIDTH_SHOULD_BE_BETWEEN_ONE_AND_ONE_HUNDRED]);
 	return;
     }
 
     if (mass < 10 || mass > 2000) {
-	Notice(w, "Mass should be between 10 and 2000");
+	Notice(w, msgText[MASS_SHOULD_BE_BETWEEN_TEN_AND_TWO_THOUSAND]);
 	return;
     }
 
     if (drag < 0 || drag > 50) {
-	Notice(w, "Drag should be between 0 and 50");
+	Notice(w, msgText[DRAG_SHOULD_BE_BETWEEN_ZERO_AND_FIFTY]);
 	return;
     }
 
@@ -240,17 +218,17 @@ static void dynPencilMenuCallback(Widget w)
     static TextPromptInfo info;
     static struct textPromptInfo value[3];
 
-    value[0].prompt = "Width:";
+    value[0].prompt = msgText[DYNAMIC_WIDTH];
     value[0].str = dynPencilWidthStr;
     value[0].len = 5;
-    value[1].prompt = "Mass:";
+    value[1].prompt = msgText[DYNAMIC_MASS];
     value[1].str = dynPencilMassStr;;
     value[1].len = 5;
-    value[2].prompt = "Drag:";
+    value[2].prompt = msgText[DYNAMIC_DRAG];
     value[2].str = dynPencilDragStr;;
     value[2].len = 5;
     info.prompts = value;
-    info.title = "Enter the desired dynamic pencil parameters";
+    info.title = msgText[ENTER_THE_DESIRED_DYNAMIC_PENCIL_PARAMETERS];
     info.nprompt = 3;
 
     TextPrompt(w, "dynpencilparams", &info, dynPencilOkCallback, NULL, NULL);
@@ -271,15 +249,15 @@ sprayOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
     int rate = atoi(info->prompts[2].rstr);
 
     if (radius < 1 || radius > 100) {
-	Notice(w, "Radius should be between 1 and 100");
+	Notice(w, msgText[RADIUS_SHOULD_BE_BETWEEN_ONE_AND_ONE_HUNDRED]);
 	return;
     }
     if (density < 1 || density > 500) {
-	Notice(w, "Density should be between 1 and 500");
+	Notice(w, msgText[DENSITY_SHOULD_BE_BETWEEN_ONE_AND_FIVE_HUNDRED]);
 	return;
     }
     if (rate < 1 || rate > 500) {
-	Notice(w, "Rate should be between 1 and 500");
+	Notice(w, msgText[RATE_SHOULD_BE_BETWEEN_ONE_AND_FIVE_HUNDRED]);
 	return;
     }
     SpraySetParameters(radius, density, rate);
@@ -295,17 +273,17 @@ sprayMenuCallback(Widget w)
     static TextPromptInfo info;
     static struct textPromptInfo value[3];
 
-    value[0].prompt = "Radius:";
+    value[0].prompt = msgText[SPRAY_RADIUS];
     value[0].str = sprayRadiusStr;
     value[0].len = 4;
-    value[1].prompt = "Density:";
+    value[1].prompt = msgText[SPRAY_DENSITY];
     value[1].str = sprayDensityStr;
     value[1].len = 4;
-    value[2].prompt = "Rate:";
+    value[2].prompt = msgText[SPRAY_RATE];
     value[2].str = sprayRateStr;
     value[2].len = 4;
     info.prompts = value;
-    info.title = "Enter the desired spray parameters";
+    info.title = msgText[ENTER_THE_DESIRED_SPRAY_PARAMETERS];
     info.nprompt = 3;
 
     TextPrompt(w, "sprayparams", &info, sprayOkCallback, NULL, NULL);
@@ -318,7 +296,7 @@ static char tfillAngleStr[10] = "0";
 static char tfillPadStr[10] = "0";
 static char tfillHOffsetStr[10] = "0";
 static char tfillVOffsetStr[10] = "0";
-static char tfillStepsStr[10] = "25";
+static char tfillStepsStr[10] = "";
 
 static void 
 tfillOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
@@ -333,23 +311,28 @@ tfillOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
     VOffset = atoi(info->prompts[3].rstr);
     Steps = atoi(info->prompts[4].rstr);
     if (HOffset < -100 || HOffset > 100) {
-	Notice(w, "Horizontal offset should be between -100 and 100 %");
+	Notice(w, msgText[
+            HORIZONTAL_OFFSET_SHOULD_BE_BETWEEN_PLUS_MINUS_HUNDRED_PERCENT]);
 	return;
     }
     if (VOffset < -100 || VOffset > 100) {
-	Notice(w, "Vertical offset should be between -100 and 100 %");
+	Notice(w, msgText[
+            VERTICAL_OFFSET_SHOULD_BE_BETWEEN_PLUS_MINUS_HUNDRED_PERCENT]);
 	return;
     }
     if (Pad < -49 || Pad > 49) {
-	Notice(w, "Pad should be between -49 and 49 %");
+	Notice(w, msgText[
+            PAD_SHOULD_BE_BETWEEN_PLUS_MINUS_FOURTY_NINE_PERCENT]);
 	return;
     }
     if (Angle < -360 || Angle > 360) {
-	Notice(w, "Angle should be between -360 and 360 degrees");
+	Notice(w, msgText[
+            ANGLE_SHOULD_BE_BETWEEN_PLUS_MINUS_THREE_HUNDRED_SIXTY_DEGREES]);
 	return;
     }
     if (Steps < 1 || Steps > 300) {
-	Notice(w, "Number of steps should be between 1 and 300");
+	Notice(w, msgText[
+            NUMBER_OF_STEPS_SHOULD_BE_BETWEEN_ONE_AND_THREE_HUNDRED]);
 	return;
     }
     TfillSetParameters(VOffset/100.0, HOffset/100.0, Pad/100.0, Angle, Steps);
@@ -367,58 +350,35 @@ tfillMenuCallback(Widget w)
     static TextPromptInfo info;
     static struct textPromptInfo value[5];
 
-    value[0].prompt = "Angle (degrees):";
+    if (!*tfillStepsStr) {
+	 if (DefaultDepthOfScreen(XtScreen(GetShell(w))) <= 8)
+	      strcpy(tfillStepsStr, "25");
+	 else
+	      strcpy(tfillStepsStr, "300");
+    }
+
+    value[0].prompt = msgText[GRADIENT_ANGLE];
     value[0].str = tfillAngleStr;
     value[0].len = 4;
-    value[1].prompt = "Pad (%):";
+    value[1].prompt = msgText[GRADIENT_PAD];
     value[1].str = tfillPadStr;
     value[1].len = 4;
-    value[2].prompt = "Horizontal offset (%):";
+    value[2].prompt = msgText[GRADIENT_HORIZONTAL_OFFSET];
     value[2].str = tfillHOffsetStr;
     value[2].len = 4;
-    value[3].prompt = "Vertical offset (%):";
+    value[3].prompt = msgText[GRADIENT_VERTICAL_OFFSET];
     value[3].str = tfillVOffsetStr;
     value[3].len = 4;
-    value[4].prompt = "Steps:";
+    value[4].prompt = msgText[GRADIENT_STEPS];
     value[4].str = tfillStepsStr;
     value[4].len = 3;
     info.prompts = value;
-    info.title = "Enter the desired gradient fill parameters";
+    info.title = msgText[ENTER_THE_DESIRED_GRADIENT_FILL_PARAMETERS];
     info.nprompt = 5;
 
     TextPrompt(w, "tfillparams", &info, tfillOkCallback, NULL, NULL);
 }
 
-/*
-**  Font menu callbacks.
- */
-static void 
-fontSetCallback(Widget paint, void *info)
-{
-    XtVaSetValues(paint, XtNfont, (XFontStruct *) info, NULL);
-    FontChanged(paint);
-}
-static void 
-fontSet(Widget w, char *name)
-{
-    XFontStruct *info;
-    Arg arg;
-
-    if (name == NULL) {
-	FontSelect(w, None);
-    } else {
-	if ((info = XLoadQueryFont(XtDisplay(GetShell(w)), name)) == NULL) {
-	    XtVaSetValues(w, XtNsensitive, False, NULL);
-	    Notice(w, "Unable to load requested font");
-	    return;
-	}
-	GraphicAll(fontSetCallback, (void *) info);
-	XtSetArg(arg, XtNfont, info);
-	OperationAddArg(arg);
-    }
-    MenuCheckItem(w, True);
-}
-
 /*
 **  Exit callback (simple)
  */
@@ -433,10 +393,12 @@ exitOkCallback(Widget w, XtPointer junk, XtPointer junk2)
     exit(0);
 #endif
 }
+
 static void 
 exitCancelCallback(Widget paint, XtPointer junk, XtPointer junk2)
 {
 }
+
 static void 
 exitPaintCheck(Widget paint, void *sumArg)
 {
@@ -445,7 +407,8 @@ exitPaintCheck(Widget paint, void *sumArg)
     XtVaGetValues(paint, XtNdirty, &flg, NULL);
     *sum += flg ? 1 : 0;
 }
-static void 
+
+void 
 exitPaint(Widget w, XtPointer junk, XtPointer junk2)
 {
     int total = 0;
@@ -456,30 +419,25 @@ exitPaint(Widget w, XtPointer junk, XtPointer junk2)
 	exitOkCallback(w, NULL, NULL);
 	return;
     }
-    AlertBox(w, "There are unsaved changes,\nare you sure you wish to quit?",
+    AlertBox(w, msgText[UNSAVED_CHANGES_WISH_TO_QUIT],
 	     exitOkCallback, exitCancelCallback, NULL);
 }
 
 /*
 **
  */
-static void 
-loadClipboardCB(Widget w, char *file, void *image)
+void 
+takeSnapshot(Widget w, XtPointer junk, XtPointer junk2)
 {
-    ClipboardSetImage(w, image);
+    SnapshotImage(GetToplevel(XtParent(w)), junk, 0);
 }
-static void 
-loadClipboard(Widget w, XtPointer junk, XtPointer junk2)
-{
-    GetFileName(GetShell(w), 0, NULL, (XtCallbackProc) loadClipboardCB, NULL);
-}
-
 
 /*
 **  Button popups 
  */
-static void brushPopupCB(Widget w, XtPointer junk, XtPointer junk2);
+void brushPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void erasePopupCB(Widget w, XtPointer junk, XtPointer junk2);
+static void arcPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void fillPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void tfillPopupCB2(Widget w, XtPointer junk, XtPointer junk2);
 #ifdef FEATURE_FRACTAL
@@ -489,99 +447,127 @@ static void selectPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void dynPencilPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void selectShapePopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void sprayPopupCB(Widget w, XtPointer junk, XtPointer junk2);
+static void linePopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void ovalPopupCB(Widget w, XtPointer junk, XtPointer junk2);
+static void splinePopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void boxPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 
 #define GENERATE_HELP(name, hlpstr)				\
 		static PaintMenuItem name [] = {		\
-			MI_SEPERATOR(),				\
+			MI_SEPARATOR(),				\
 			MI_SIMPLECB("help", HelpDialog, hlpstr)	\
 		};
 
 static PaintMenuItem brushPopup[] =
 {
-    MI_SEPERATOR(),
-    MI_FLAGCB("opaque", MF_CHECKON | MF_GROUP1, brushPopupCB, 0),
+    MI_SEPARATOR(),
+    MI_FLAGCB("opaque", MF_CHECKON | MF_GROUP1, brushPopupCB, 0), 
     MI_FLAGCB("transparent", MF_CHECK | MF_GROUP1, brushPopupCB, 1),
+    MI_FLAGCB("stain", MF_CHECK | MF_GROUP1, brushPopupCB, 2),
     MI_SIMPLECB("select", BrushSelect, NULL),
     MI_SIMPLECB("parms", changeBrushParmAction, NULL),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.brush"),
 };
 static PaintMenuItem erasePopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("original", MF_CHECKON, erasePopupCB, 0),
     MI_SIMPLECB("select", changeBrushAction, NULL),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.erase"),
 };
 
 static PaintMenuItem dynPencilPopup[] = 
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("autofinish", MF_CHECK, dynPencilPopupCB, 0),
     MI_SIMPLECB("select", changeDynPencilAction, NULL),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.dynpencil"),
 };
 
 static PaintMenuItem sprayPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("gauss", MF_CHECKON, sprayPopupCB, 0),
     MI_SIMPLECB("select", changeSprayAction, NULL),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.spray"),
 };
 
 static PaintMenuItem smearPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_SIMPLECB("select", BrushSelect, NULL),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.smear"),
 };
 
+static PaintMenuItem linePopup[] =
+{
+    MI_SEPARATOR(),
+    MI_FLAGCB("ray", MF_CHECK, linePopupCB, 0),
+    MI_FLAGCB("arrow", MF_CHECK, linePopupCB, 1),
+    MI_SEPARATOR(),
+    MI_SIMPLECB("help", HelpDialog, "toolbox.tools.line"),
+};
+
 GENERATE_HELP(pencilPopup, "toolbox.tools.pencil")
 GENERATE_HELP(dotPencilPopup, "toolbox.tools.dotPencil")
-GENERATE_HELP(linePopup, "toolbox.tools.line")
-GENERATE_HELP(rayPopup, "toolbox.tools.ray")
-GENERATE_HELP(arcPopup, "toolbox.tools.arc")
+
+static PaintMenuItem arcPopup[] =
+{
+    MI_SEPARATOR(),
+    MI_FLAGCB("connect", MF_CHECKON | MF_GROUP1, arcPopupCB, 0),
+    MI_FLAGCB("quadrant", MF_CHECK | MF_GROUP1, arcPopupCB, 1),
+    MI_FLAGCB("centered", MF_CHECK | MF_GROUP1, arcPopupCB, 2),
+    MI_FLAGCB("boxed", MF_CHECK | MF_GROUP1, arcPopupCB, 3),
+    MI_SEPARATOR(),
+    MI_SIMPLECB("help", HelpDialog, "toolbox.tools.arc"),
+};
 static PaintMenuItem fillPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("fill", MF_CHECKON | MF_GROUP1, fillPopupCB, 0),
     MI_FLAGCB("change", MF_CHECK | MF_GROUP1, fillPopupCB, 1),
     MI_FLAGCB("fill_range", MF_CHECK | MF_GROUP1, fillPopupCB, 2),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.fill"),
 };
 static PaintMenuItem tfillPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("fill", MF_CHECKON | MF_GROUP1, fillPopupCB, 0),
     MI_FLAGCB("change", MF_CHECK | MF_GROUP1, fillPopupCB, 1),
     MI_FLAGCB("fill_range", MF_CHECK | MF_GROUP1, fillPopupCB, 2),
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("radial", MF_CHECKON | MF_GROUP2, tfillPopupCB2, TFILL_RADIAL),
     MI_FLAGCB("linear", MF_CHECK | MF_GROUP2, tfillPopupCB2, TFILL_LINEAR),
     MI_FLAGCB("conical", MF_CHECK | MF_GROUP2, tfillPopupCB2, TFILL_CONE),
     MI_FLAGCB("square", MF_CHECK | MF_GROUP2, tfillPopupCB2, TFILL_SQUARE),
     MI_SIMPLECB("parms", changeTFillAction, NULL),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.tfill"),
 };
 #ifdef FEATURE_FRACTAL
 static PaintMenuItem ffillPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("fill", MF_CHECKON | MF_GROUP1, fillPopupCB, 0),
     MI_FLAGCB("change", MF_CHECK | MF_GROUP1, fillPopupCB, 1),
     MI_FLAGCB("fill_range", MF_CHECK | MF_GROUP1, fillPopupCB, 2),
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("plasma", MF_CHECKON | MF_GROUP2, ffillPopupCB2, 0),
     MI_FLAGCB("clouds", MF_CHECK | MF_GROUP2, ffillPopupCB2, 1),
     MI_FLAGCB("landscape", MF_CHECK | MF_GROUP2, ffillPopupCB2, 2),
+    MI_SIMPLECB("parms", changeFractalParmAction, NULL),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.ffill"),
 };
 #endif
 static PaintMenuItem textPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_SIMPLECB("select", changeFontAction, NULL),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.text"),
 };
@@ -590,59 +576,78 @@ GENERATE_HELP(freehandPopup, "toolbox.tools.blob")
 GENERATE_HELP(clinePopup, "toolbox.tools.cline")
 static PaintMenuItem selectBPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("shape", MF_CHECKON | MF_GROUP1, selectPopupCB, 0),
     MI_FLAGCB("not_color", MF_CHECK | MF_GROUP1, selectPopupCB, 1),
     MI_FLAGCB("only_color", MF_CHECK | MF_GROUP1, selectPopupCB, 2),
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("rectangle", MF_CHECKON | MF_GROUP2, selectShapePopupCB, 0),
     MI_FLAGCB("ellipse", MF_CHECK | MF_GROUP2, selectShapePopupCB, 1),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.select"),
 };
 static PaintMenuItem lassoPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("shape", MF_CHECKON | MF_GROUP1, selectPopupCB, 0),
     MI_FLAGCB("not_color", MF_CHECK | MF_GROUP1, selectPopupCB, 1),
     MI_FLAGCB("only_color", MF_CHECK | MF_GROUP1, selectPopupCB, 2),
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.select"),
 };
 static PaintMenuItem selectAPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("shape", MF_CHECKON | MF_GROUP1, selectPopupCB, 0),
     MI_FLAGCB("not_color", MF_CHECK | MF_GROUP1, selectPopupCB, 1),
     MI_FLAGCB("only_color", MF_CHECK | MF_GROUP1, selectPopupCB, 2),
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.select"),
 };
 static PaintMenuItem boxPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("center", MF_CHECK, boxPopupCB, 0),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.box"),
 };
 static PaintMenuItem fboxPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("center", MF_CHECK, boxPopupCB, 0),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.box"),
 };
 static PaintMenuItem ovalPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("center", MF_CHECK, ovalPopupCB, 0),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.oval"),
 };
 static PaintMenuItem fovalPopup[] =
 {
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
     MI_FLAGCB("center", MF_CHECK, ovalPopupCB, 0),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.oval"),
 };
+static PaintMenuItem splinePopup[] =
+{
+    MI_SEPARATOR(),
+    MI_FLAGCB("open", MF_CHECKON | MF_GROUP1, splinePopupCB, 0),
+    MI_FLAGCB("closed", MF_CHECK | MF_GROUP1, splinePopupCB, 1),
+    MI_FLAGCB("closed_up", MF_CHECK | MF_GROUP1, splinePopupCB, 2),
+    MI_SEPARATOR(),
+    MI_SIMPLECB("help", HelpDialog, "toolbox.tools.spline"),
+};
+static PaintMenuItem fsplinePopup[] =
+{
+    MI_SEPARATOR(),
+    MI_FLAGCB("open", MF_CHECKON | MF_GROUP1, splinePopupCB, 0),
+    MI_FLAGCB("closed", MF_CHECK | MF_GROUP1, splinePopupCB, 1),
+    MI_FLAGCB("closed_up", MF_CHECK | MF_GROUP1, splinePopupCB, 2),
+    MI_SEPARATOR(),
+    MI_SIMPLECB("help", HelpDialog, "toolbox.tools.spline"),
+};
 
-static void 
+void 
 brushPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
     int nm = (int) junk;
@@ -660,6 +665,17 @@ ovalPopupCB(Widget w, XtPointer junk, XtPointer junk2)
     MenuCheckItem(ovalPopup[1].widget, m);
     MenuCheckItem(fovalPopup[1].widget, m);
 }
+
+static void 
+splinePopupCB(Widget w, XtPointer junk, XtPointer junk2)
+{
+    int nm = (int) junk;
+
+    MenuCheckItem(splinePopup[nm + 1].widget, True);
+    MenuCheckItem(fsplinePopup[nm + 1].widget, True);
+    SplineSetMode(nm);
+}
+
 static void 
 boxPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
@@ -669,6 +685,16 @@ boxPopupCB(Widget w, XtPointer junk, XtPointer junk2)
     MenuCheckItem(boxPopup[1].widget, m);
     MenuCheckItem(fboxPopup[1].widget, m);
 }
+
+static void 
+arcPopupCB(Widget w, XtPointer junk, XtPointer junk2)
+{
+    int nm = (int) junk;
+
+    ArcSetMode(nm);
+    MenuCheckItem(w, True);
+}
+
 static void 
 fillPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
@@ -740,6 +766,23 @@ sprayPopupCB(Widget w, XtPointer junk, XtPointer junk2)
     MenuCheckItem(w, m);
 }
 static void 
+linePopupCB(Widget w, XtPointer junk, XtPointer junk2)
+{
+    int nm = (int) junk;
+    Boolean m;
+
+    if (nm == 0) {
+         m = !RayGetStyle();
+         RaySetStyle(m);
+         MenuCheckItem(linePopup[1].widget, m);
+    }
+    if (nm == 1) {
+         m = !ArrowGetStyle();
+         ArrowSetStyle(m);
+         MenuCheckItem(linePopup[2].widget, m);
+    }
+}
+static void 
 erasePopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
     Boolean m = !EraseGetMode();
@@ -763,7 +806,6 @@ GENERATE_OP(DynPencil)
 GENERATE_OP(Box)
 GENERATE_OP(FBox)
 GENERATE_OP(Line)
-GENERATE_OP(Ray)
 GENERATE_OP(Arc)
 GENERATE_OP(Oval)
 GENERATE_OP(FOval)
@@ -779,9 +821,12 @@ GENERATE_OP(FFreehand)
 GENERATE_OP(CLine)
 GENERATE_OP(Fill)
 GENERATE_OP(TFill)
+GENERATE_OP(Spline)
+GENERATE_OP(FSpline)
 #ifdef FEATURE_FRACTAL
 GENERATE_OP(FFill)
 #endif
+
 GENERATE_OP(SelectBox)
 GENERATE_OP(Lasso)
 GENERATE_OP(SelectPoly)
@@ -801,24 +846,57 @@ typedef struct {
 static Widget iconListWidget;
 static IconListItem iconList[] =
 {
-    {"brush", XPMMAP(brush), &BrushOp,
-     "<BtnDown>(2): changeBrush()", MENU(brush), None},
-    {"erase", XPMMAP(erase), &EraseOp,
-     "<BtnDown>(2): changeBrush()", MENU(erase), None},
+    /* Vertical arrangement */
+
     {"pencil", XPMMAP(pencil), &PencilOp,
      NULL, MENU(pencil), None},
     {"dynpencil", XPMMAP(dynPen), &DynPencilOp, 	
      "<BtnDown>(2): changeDynPencil()", MENU(dynPencil), None},
     {"dotPencil", XPMMAP(dotPen), &DotPencilOp,
      NULL, MENU(dotPencil), None},
-    {"selectBox", XPMMAP(select), &SelectBoxOp,
-     NULL, MENU(selectB), None},
-    {"lasso", XPMMAP(lasso), &LassoOp,
-     NULL, MENU(lasso), None},
-    {"selectPoly", XPMMAP(selpoly), &SelectPolyOp,
-     NULL, MENU(selectA), None},
+
+    {"brush", XPMMAP(brush), &BrushOp,
+     "<BtnDown>(2): changeBrush()", MENU(brush), None},
     {"spray", XPMMAP(spray), &SprayOp,
      "<BtnDown>(2): changeSpray()", MENU(spray), None},
+    {"smear", XPMMAP(smear), &SmearOp,
+     "<BtnDown>(2): changeBrush()", MENU(smear), None},
+
+    {"box", XPMMAP(box), &BoxOp,
+     NULL, MENU(box), None},
+    {"fBox", XPMMAP(fbox), &FBoxOp,
+     NULL, MENU(fbox), None},
+    {"line", XPMMAP(line), &LineOp,
+     NULL, MENU(line), None},
+
+    {"oval", XPMMAP(oval), &OvalOp,
+     NULL, MENU(oval), None},
+    {"fOval", XPMMAP(foval), &FOvalOp,
+     NULL, MENU(foval), None},
+    {"arc", XPMMAP(arc), &ArcOp,
+     NULL, MENU(arc), None},
+
+    {"polygon", XPMMAP(poly), &PolyOp,
+     NULL, MENU(poly), None},
+    {"fPolygon", XPMMAP(fpoly), &FPolyOp,
+     NULL, MENU(poly), None},
+    {"cLine", XPMMAP(cline), &CLineOp,
+     NULL, MENU(cline), None},
+
+    {"Spline", XPMMAP(spline), &SplineOp,
+     NULL, MENU(spline), None},
+    {"fSpline", XPMMAP(fspline), &FSplineOp,
+     NULL, MENU(fspline), None},
+    {"text", XPMMAP(text), &FontOp,
+     "<BtnDown>(2): changeFont()", MENU(text), None},
+
+    {"freehand", XPMMAP(freehand), &FreehandOp,
+     NULL, MENU(freehand), None},
+    {"fFreehand", XPMMAP(ffreehand), &FFreehandOp,
+     NULL, MENU(freehand), None},
+    {"erase", XPMMAP(erase), &EraseOp,
+     "<BtnDown>(2): changeBrush()", MENU(erase), None},
+
     {"fill", XPMMAP(fill), &FillOp,
      NULL, MENU(fill), None},
     {"tfill", XPMMAP(tfill), &TFillOp,
@@ -827,36 +905,41 @@ static IconListItem iconList[] =
     {"ffill", XPMMAP(ffill), &FFillOp,
      NULL, MENU(ffill), None},
 #endif
-    {"smear", XPMMAP(smear), &SmearOp,
-     "<BtnDown>(2): changeBrush()", MENU(smear), None},
-    {"text", XPMMAP(text), &FontOp,
-     "<BtnDown>(2): changeFont()", MENU(text), None},
-    {"line", XPMMAP(line), &LineOp,
-     NULL, MENU(line), None},
-    {"ray", XPMMAP(ray), &RayOp,
-     NULL, MENU(ray), None},
-    {"arc", XPMMAP(arc), &ArcOp,
-     NULL, MENU(arc), None},
-    {"cLine", XPMMAP(cline), &CLineOp,
-     NULL, MENU(cline), None},
-    {"box", XPMMAP(box), &BoxOp,
-     NULL, MENU(box), None},
-    {"fBox", XPMMAP(fbox), &FBoxOp,
-     NULL, MENU(fbox), None},
-    {"oval", XPMMAP(oval), &OvalOp,
-     NULL, MENU(oval), None},
-    {"fOval", XPMMAP(foval), &FOvalOp,
-     NULL, MENU(foval), None},
-    {"polygon", XPMMAP(poly), &PolyOp,
-     NULL, MENU(poly), None},
-    {"fPolygon", XPMMAP(fpoly), &FPolyOp,
-     NULL, MENU(poly), None},
-    {"freehand", XPMMAP(freehand), &FreehandOp,
-     NULL, MENU(freehand), None},
-    {"fFreehand", XPMMAP(ffreehand), &FFreehandOp,
-     NULL, MENU(freehand), None},
+
+    {"selectBox", XPMMAP(select), &SelectBoxOp,
+     NULL, MENU(selectB), None},
+    {"lasso", XPMMAP(lasso), &LassoOp,
+     NULL, MENU(lasso), None},
+    {"selectPoly", XPMMAP(selpoly), &SelectPolyOp,
+     NULL, MENU(selectA), None},
+
+    /* Vertical arrangement */
 };
 
+static int 
+getIndexOp()
+{
+    int i;
+    for (i=0; i<XtNumber(iconList); i++)
+         if (CurrentOp == iconList[i].data) return i;
+    return 0;
+}
+
+static void
+permuteIcons()
+{
+#define NUM XtNumber(iconList)
+    int i;
+    IconListItem tempList[NUM];
+
+    if (ToolsAreHorizontal()) {
+    for (i=0; i<NUM; i++)
+        tempList[i] = iconList[3*(i%9)+(i/9)];
+    for (i=0; i<NUM; i++)
+        iconList[i] = tempList[i];
+    }
+}
+
 void 
 OperationSet(String names[], int num)
 {
@@ -877,6 +960,7 @@ OperationSet(String names[], int num)
 	XawToggleSetCurrent(iconListWidget, (XtPointer) match->name);
 }
 
+#if 0
 static PaintMenuItem lineMenu[] =
 {
 #if 0
@@ -913,10 +997,11 @@ static PaintMenuItem fontMenu[] =
 	      fontSet, "-*-fixed-medium-r-normal-*-*-120-*-*-m-*-*-*"),
     MI_FLAGCB("Courier 12", MF_CHECK | MF_GROUP1,
 	      fontSet, "-*-courier-medium-r-normal-*-*-120-*-*-m-*-*-*"),
-    MI_SEPERATOR(),
+    MI_SEPARATOR(),
 #define FM_SELECT	11
     MI_FLAGCB("select", MF_CHECK | MF_GROUP1, fontSet, NULL),
 };
+#endif
 
 #if 0
 static PaintMenuItem otherMenu[] =
@@ -927,31 +1012,34 @@ static PaintMenuItem otherMenu[] =
 };
 #endif
 
-static PaintMenuItem fileMenu[] =
+static PaintMenuItem canvasMenu[] =
 {
     MI_SIMPLECB("new", GraphicCreate, 0),
     MI_SIMPLECB("new-size", GraphicCreate, 2),
     MI_SIMPLECB("open", GraphicCreate, 1),
-    MI_SIMPLECB("load-clip", loadClipboard, NULL),
-    MI_SEPERATOR(),
+    MI_SIMPLECB("snapshot", takeSnapshot, NULL),
+    MI_SEPARATOR(),
     MI_SIMPLECB("quit", exitPaint, NULL),
 };
 
 static PaintMenuItem helpMenu[] =
 {
+    MI_SIMPLECB("intro", HelpDialog, "introduction"),
+    MI_SIMPLECB("tools", HelpDialog, "toolbox.tools"),
+    MI_SIMPLECB("canvas", HelpDialog, "canvas"),
+    MI_SEPARATOR(),
     MI_SIMPLECB("about", HelpDialog, "about"),
-    MI_SEPERATOR(),
-    MI_SIMPLECB("help", HelpDialog, "toolbox.toolbox"),
+    MI_SIMPLECB("copyright", HelpDialog, "copyright"),
 };
 
 static PaintMenuBar menuBar[] =
 {
-    {None, "file", XtNumber(fileMenu), fileMenu},
+    {None, "canvas", XtNumber(canvasMenu), canvasMenu},
 #if 0
     {None, "other", XtNumber(otherMenu), otherMenu},
-#endif
     {None, "line", XtNumber(lineMenu), lineMenu},
     {None, "font", XtNumber(fontMenu), fontMenu},
+#endif
     {None, "help", XtNumber(helpMenu), helpMenu},
 };
 
@@ -991,11 +1079,12 @@ OperationAddArg(Arg arg)
 /*
 **  Double click action callback functions.
  */
-static void 
-changeLineAction(Widget w, XEvent * event)
+void 
+changeFontAction(Widget w, XEvent * event)
 {
-    lineWidth(lineMenu[LW_SELECT].widget, (int) lineMenu[LW_SELECT].data);
+    FontSelect(w, None);
 }
+
 static void
 changeDynPencilAction(Widget w, XEvent *event)
 {
@@ -1021,11 +1110,39 @@ changeBrushParmAction(Widget w, XEvent * event)
 {
     brushMenuCallback(w);
 }
+static void 
+changeFractalParmAction(Widget w, XEvent * event)
+{
+    fractalMenuCallback(w);
+}
 
 static void 
-changeFontAction(Widget w, XEvent * event)
+switchtoCanvasCallback(Widget w, XtPointer junk, XEvent * event, Boolean * flg)
 {
-    fontSet(fontMenu[FM_SELECT].widget, (char *) fontMenu[FM_SELECT].data);
+    if (Global.canvas && event->type == ButtonRelease)
+       XMapRaised(XtDisplay(Global.canvas), XtWindow(Global.canvas));
+}
+
+void
+toolsResized(Widget w, XtPointer l, XConfigureEvent * event, Boolean * flg)
+{
+    Dimension width, height;
+    XtVaGetValues(w, XtNwidth, &width,
+		     XtNheight, &height, NULL);
+    XtResizeWidget((Widget)l, (width>20)? width - 8:12 ,
+                                  (height>50)? height - 38:12,
+#ifdef XAW3D		   
+		                  1);
+    XtMoveWidget((Widget)l, 4, 35);
+#else
+		                  0);
+    XtMoveWidget((Widget)l, 4, 36);
+#endif		
+#if defined(XAWPLAIN) || defined(XAW95) || defined(XAW3DG)
+    XtMoveWidget(Global.back, width-20, 8);
+#else
+    XtMoveWidget(Global.back, width-15, 8);
+#endif
 }
 
 /*
@@ -1036,7 +1153,7 @@ OperationInit(Widget toplevel)
 {
     static XtActionsRec acts[] =
     {
-	{"changeLine", (XtActionProc) changeLineAction},
+      /*	{"changeLine", (XtActionProc) changeLineAction}, */
 	{"changeBrush", (XtActionProc) changeBrushAction},
 	{"changeFont", (XtActionProc) changeFontAction},
 	{"changeDynPencil", (XtActionProc) changeDynPencilAction},
@@ -1045,17 +1162,15 @@ OperationInit(Widget toplevel)
     };
     int i;
     Pixmap pix;
-    Widget vport, form, box, icon, bar, firstIcon = None;
+    Widget form, vport, box, icon, firstIcon = None;
     char *defTrans = "<BtnDown>,<BtnUp>: set() notify()\n";
     XtTranslations trans = XtParseTranslationTable(defTrans);
     IconListItem *cur;
     XpmAttributes attributes;
 
-    
     form = XtVaCreateManagedWidget("toolbox",
 				   formWidgetClass, toplevel,
 				   XtNborderWidth, 0,
-				   XtNwidth, 300,
 				   NULL);
     XtAppAddActions(XtWidgetToApplicationContext(toplevel),
 		    acts, XtNumber(acts));
@@ -1063,8 +1178,20 @@ OperationInit(Widget toplevel)
     /*
     **  Create the menu bar
      */
-    bar = MenuBarCreate(form, XtNumber(menuBar), menuBar);
-
+    Global.bar = MenuBarCreate(form, XtNumber(menuBar), menuBar);
+    XtVaSetValues(Global.bar, 
+#ifdef XAW3D
+#ifdef XAW3DG
+                  XtNhorizDistance, 0,
+#else
+                  XtNhorizDistance, -2,
+#endif
+		  XtNvertDistance, 3,
+#else
+                  XtNhorizDistance, 0,
+#endif		  
+		  NULL);
+    
     /*
     **  Create the operation icon list
      */
@@ -1072,13 +1199,38 @@ OperationInit(Widget toplevel)
 				    viewportWidgetClass, form,
 				    XtNallowVert, True,
 				    XtNuseRight, True,
-				    XtNfromVert, bar,
+				    XtNfromVert, Global.bar,
+				    XtNborderWidth, 0,
+#ifdef XAW95
+				    XtNvertDistance, 3,
+#endif				    	       
+#ifdef XAW3D
+				    XtNvertDistance, 5,
+#endif				    
 				    NULL);
     box = XtVaCreateManagedWidget("box",
-				  boxWidgetClass, vport,
-				  XtNwidth, 32 * 8,
-				  XtNtop, XtChainTop,
-				  NULL);
+				    boxWidgetClass, vport,
+#ifdef XAW3D
+                                    XtNbackground, 0xc0c0c0,
+ 			            XtNwidth, 507, XtNheight, 507,
+#endif				  
+				    XtNtop, XtChainTop,
+				    NULL);
+
+    Global.back = XtVaCreateManagedWidget("!", labelWidgetClass, form,
+				      XtNwidth, 12,
+				      XtNvertDistance, 8,
+                                      XtNborderWidth, 1,
+                                      XtNsensitive, False,
+                                      NULL);
+
+    XtAddEventHandler(Global.back, ButtonPressMask | ButtonReleaseMask, False,
+        (XtEventHandler) switchtoCanvasCallback, NULL);
+
+    permuteIcons();
+
+    attributes.valuemask = XpmCloseness;
+    attributes.closeness =  40000;
 
     for (i = 0; i < XtNumber(iconList); i++) {
 	cur = &iconList[i];
@@ -1091,8 +1243,6 @@ OperationInit(Widget toplevel)
 				       NULL);
 
 	if (cur->xpmmap != NULL) {
-	    attributes.valuemask = XpmCloseness;
-            attributes.closeness =  40000;
 	    XpmCreatePixmapFromData(XtDisplay(box),
 				    DefaultRootWindow(XtDisplay(box)),
 				    cur->xpmmap,
@@ -1102,8 +1252,8 @@ OperationInit(Widget toplevel)
 	}
 
 	if (pix == None) {
-	    printf("Unable to allocate sufficient color entries.\n");
-	    printf("Try exiting other color intensive applications.\n");
+	    fprintf(stderr, msgText[UNABLE_TO_ALLOCATE_SUFFICIENT_COLOR_ENTRIES]);
+	    fprintf(stderr, msgText[TRY_EXITING_OTHER_COLOR_INTENSIVE_APPLICATIONS]);
 	    exit(1);
 	}
 	
@@ -1126,15 +1276,80 @@ OperationInit(Widget toplevel)
 	}
 	XtAddCallback(icon, XtNcallback,
 		      setOperation, cur->data);
+#ifdef XAW3D
+        XtVaSetValues(icon, XtNwidth, 32, XtNheight, 32,
+                            XtNforeground, 0xc0c0c0, NULL);
+#endif				       
 
 	if (cur->nitems != 0 && cur->popupMenu != NULL)
-	    MenuPopupCreate(icon, cur->nitems, cur->popupMenu);
+	    MenuPopupCreate(icon, "popup-menu", cur->nitems, cur->popupMenu);
     }
 
-    lineWidth(lineMenu[0].widget, (int) lineMenu[0].data);
-    fontSet(fontMenu[1].widget, fontMenu[1].data);
-
+    if (ToolsAreHorizontal())
+        XtVaSetValues(vport,
+#if defined(XAW3D) && !defined(XAW3DG)		      
+#ifdef BIGTOOLICONS
+	          XtNwidth, 507, XtNheight, 160,
+#else
+	          XtNwidth, 328, XtNheight, 108,
+#endif
+#endif
+#ifdef XAW3DG
+#ifdef BIGTOOLICONS
+	          XtNwidth, 507, XtNheight, 160,
+#else
+	          XtNwidth, 332, XtNheight, 112,
+#endif
+#endif
+#ifdef XAW95
+#ifdef BIGTOOLICONS
+	          XtNwidth, 496, XtNheight, 157,
+#else
+	          XtNwidth, 351, XtNheight, 107,
+#endif
+#endif
+#ifdef XAWPLAIN		      
+#ifdef BIGTOOLICONS
+		  XtNwidth, 490, XtNheight, 154,
+#else
+		  XtNwidth, 346, XtNheight, 106,
+#endif
+#endif	      
+	         NULL);   
+    else
+        XtVaSetValues(vport,
+#if defined(XAW3D) && !defined(XAW3DG)
+#ifdef BIGTOOLICONS
+	          XtNwidth, 171, XtNheight, 468,
+#else
+	          XtNwidth, 113, XtNheight, 324,
+#endif
+#endif
+#ifdef XAW3DG
+#ifdef BIGTOOLICONS
+	          XtNwidth, 164, XtNheight, 439,
+#else
+	          XtNwidth, 116, XtNheight, 328,
+#endif
+#endif
+#ifdef XAW95
+#ifdef BIGTOOLICONS
+	          XtNwidth, 171, XtNheight, 457,
+#else
+	          XtNwidth, 123, XtNheight, 310,
+#endif
+#endif
+#ifdef XAWPLAIN		      
+#ifdef BIGTOOLICONS
+		  XtNwidth, 166, XtNheight, 454,
+#else
+	          XtNwidth, 118, XtNheight, 310,
+#endif
+#endif	      
+	         NULL);   
     AddDestroyCallback(toplevel, (DestroyCallbackFunc) exitPaint, NULL);
+    XtAddEventHandler(toplevel, StructureNotifyMask, False,
+		      (XtEventHandler) toolsResized, (XtPointer) vport);
 }
 
 Image *
@@ -1150,3 +1365,9 @@ OperationIconImage(Widget w, char *name)
 
     return PixmapToImage(w, iconList[i].icon, None);
 }
+
+void setToolIconOnWidget(Widget w)
+{
+    if (w != None)
+        XtVaSetValues(w, XtNbitmap, iconList[getIndexOp()].icon, NULL);
+}
