@@ -12,12 +12,12 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: writeGIF.c,v 1.11 2000/10/02 20:53:00 torsten Exp $ */
+/* $Id: writeGIF.c,v 1.21 2005/03/20 20:15:34 demailly Exp $ */
 
 /* ppmtogif.c - read a portable pixmap and produce a GIF file
 **
-** Based on GIFENCOD by David Rowley <mgardi@watdscu.waterloo.edu>.A
-** Lempel-Ziv compression based on "compress".
+** Based on GIFENCOD by David Rowley <mgardi@watdscu.waterloo.edu>.
+** A Lempel-Ziv compression based on "compress".
 **
 ** Copyright (C) 1989 by Jef Poskanzer.
 **
@@ -38,11 +38,11 @@
 #endif
 
 #include <stdio.h>
+#include <errno.h>
+
 #include "image.h"
 #include "rwTable.h"
-#ifdef __EMX__
-#include <errno.h>
-#endif
+
 
 #define MAXCOLORS 256
 
