@@ -15,27 +15,32 @@
 /* Polygon selection code and other portions */
 /* Copyright 1995, 1996 Torsten Martinsen    */
 
-/* $Id: selectOp.c,v 1.7 1997/07/14 17:37:11 torsten Exp $ */
+/* $Id: selectOp.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
+
+#include <stdio.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
 #include <X11/cursorfont.h>
-#include <X11/Xaw/Form.h>
-#include <X11/Xaw/AsciiText.h>
-#include <X11/Xaw/Toggle.h>
-#include <X11/Xaw/Scrollbar.h>
-#include <X11/Xaw/Command.h>
 #include <X11/Shell.h>
 #include <X11/keysym.h>
-#include <stdio.h>
+
+#include "xaw_incdir/Form.h"
+#include "xaw_incdir/AsciiText.h"
+#include "xaw_incdir/Toggle.h"
+#include "xaw_incdir/Scrollbar.h"
+#include "xaw_incdir/Command.h"
+
 #include "xpaint.h"
 #include "Paint.h"
+#include "PaintP.h"
 #include "protocol.h"
 #include "palette.h"
 #include "color.h"
 #include "misc.h"
 #include "operation.h"
 #include "ops.h"
+#include "region.h"
 
 /*
 **  It is now historical, that the Box and Area selection
@@ -53,7 +58,7 @@ typedef struct {
     Widget w;
     Drawable drawable;
 
-    Boolean drawn;
+    int drawn;
 
     int wx, wy;
 
@@ -84,6 +89,8 @@ typedef struct {
     XRectangle pixBox;
 } LocalInfo;
 
+static Boolean chromaCut(Widget w, LocalInfo * l, Pixmap * mask);
+
 #define MKRECT(rect, sx, sy, ex, ey, typeFlag) do {			\
 	if (typeFlag) {		/* square */				\
 		(rect)->width  = MIN(ABS(sx - ex),ABS(sy - ey));	\
@@ -98,7 +105,27 @@ typedef struct {
 	}								\
 } while (0)
 
-static Boolean chromaCut(Widget w, LocalInfo * l, Pixmap * mask);
+/* 
+ * If a region is already visible, unselect everything !!
+ */
+
+static int
+checkVisibility(Widget w, LocalInfo * l, XButtonEvent * event)
+{
+    PaintWidget pw = (PaintWidget) w;
+    if (pw->paint.region.isVisible) {
+        PwRegionFinish(w, True);
+        pw->paint.region.isVisible = False;
+        l->drawn = False;
+        l->tracking = False;
+	l->first = False;
+	l->npoint = 0;
+	l->endX = l->startX = l->lastX = l->wx = event->x;
+	l->endY = l->startY = l->lastY = l->wy = event->y;
+	return 1;
+    } else
+        return 0;
+}
 
 /*
 **  press
@@ -109,8 +136,15 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     /*
     **  Check to make sure all buttons are up, before doing this
      */
+
+    if (event->button >= Button4) return;    
     if ((event->state & AllButtonsMask) != 0)
 	return;
+    if (event->button == Button3) return;
+
+    if (l->first) {
+        if (checkVisibility(w, l, event)) return;
+    }
 
     if (info->surface == opWindow) {
 	l->w = w;
@@ -133,6 +167,7 @@ press(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     l->window_p[0].x = l->wx - info->zoom / 2;
     l->window_p[0].y = l->wy - info->zoom / 2;
     l->npoint = 1;
+    l->tracking = True;
 }
 
 static void 
@@ -158,6 +193,11 @@ keyPress(Widget w, LocalInfo * l, XKeyEvent * event, OpInfo * info)
     case XK_Right:
 	dx = 1;
 	break;
+    case XK_space:
+        Global.transparent = !Global.transparent;
+        PwRegionTear(w);
+	RegionTransparency((PaintWidget) w);
+	break;
     default:
 	break;
     }
@@ -180,6 +220,18 @@ keyPress(Widget w, LocalInfo * l, XKeyEvent * event, OpInfo * info)
     }
 }
 
+static void 
+keyRelease(Widget w, LocalInfo * l, XKeyEvent * event, OpInfo * info)
+{
+    KeySym keysym;
+    int len;
+    char buf[21];
+
+    len = XLookupString(event, buf, sizeof(buf) - 1, &keysym, NULL);
+
+    /* Noop at this time */
+}
+
 static void 
 finishPolyBand(Widget w, LocalInfo * l, OpInfo * info)
 {
@@ -261,6 +313,9 @@ finishPolyBand(Widget w, LocalInfo * l, OpInfo * info)
 static void 
 pressPolyBand(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 {
+    if (l->first) {
+        if (checkVisibility(w, l, event)) return;
+    }
     if (!l->tracking && (event->button == Button1)) {
 	l->endX = l->startX = event->x;
 	l->endY = l->startY = event->y;
@@ -276,9 +331,8 @@ pressPolyBand(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
 	l->drawn = False;
 	l->tracking = True;
 	l->first = True;
-    } else if ((event->button == Button2) && l->tracking) {
+    } else if (l->tracking && event->button == Button2) {
 	finishPolyBand(w, l, info);
-	return;
     }
 }
 
@@ -290,6 +344,9 @@ motionBoxBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 {
     XRectangle rect;
 
+    if (l->npoint == -1 || !l->tracking) 
+        return;
+
     if (l->drawn) {
 	MKRECT(&rect, l->window_p[0].x, l->window_p[0].y,
 	       l->lastX, l->lastY, l->lastType);
@@ -318,6 +375,10 @@ motionBoxBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 static void 
 motionLassoBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 {
+
+    if (l->npoint == -1 || !l->tracking)  
+        return;
+
     if (l->npoint != 0 && (l->real_p[l->npoint - 1].x == info->x) &&
 	(l->real_p[l->npoint - 1].y == info->y))
 	return;
@@ -351,8 +412,9 @@ motionLassoBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 static void 
 motionPolyBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 {
-    if (!l->tracking)
-	return;
+
+    if (l->npoint == -1 || !l->tracking)
+        return;
 
     if (l->drawn)
 	XDrawLine(XtDisplay(w), info->drawable, l->gcx,
@@ -369,6 +431,10 @@ motionPolyBand(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 static void 
 motion(Widget w, LocalInfo * l, XMotionEvent * event, OpInfo * info)
 {
+
+    if (l->npoint == -1) 
+        return;
+
     if (!event->state)
 	return;
 
@@ -383,7 +449,7 @@ chromaCut(Widget w, LocalInfo * l, Pixmap * mask)
 {
     GC gc;
     XImage *src, *mimg;
-    XRectangle *rect = PwScaleRectangle(w, &l->pixBox);
+    XRectangle *rect;
     int x, y, count = 0;
     Pixel p;
     XColor *xcol;
@@ -395,6 +461,15 @@ chromaCut(Widget w, LocalInfo * l, Pixmap * mask)
     Colormap cmap;
     int step;
 
+    if (l)
+        rect = PwScaleRectangle(w, &l->pixBox);
+    else {
+        rect = (XRectangle *) xmalloc(sizeof(XRectangle));
+	rect->x = 0;
+	rect->y = 0;
+	XtVaGetValues(w, XtNdrawWidth, &rect->width, 
+                         XtNdrawHeight, &rect->height, NULL);
+    }
     src = PwGetImage(w, rect);
     XtVaGetValues(GetShell(w), XtNcolormap, &cmap, NULL);
     map = PaletteFind(w, cmap);
@@ -467,6 +542,9 @@ releaseBoxBand(Widget w, LocalInfo * l, XButtonEvent * event,
     XRectangle rect;
     int width, height;
     GC gc;
+
+    l->tracking = False;
+
     if (l->drawn) {
 	MKRECT(&rect, l->window_p[0].x, l->window_p[0].y,
 	       l->lastX, l->lastY, l->lastType);
@@ -521,6 +599,8 @@ releaseLassoBand(Widget w, LocalInfo * l, XButtonEvent * event,
     GC gc;
     XPoint tmp;
 
+    l->tracking = False;
+
     XDrawLines(XtDisplay(w), XtWindow(w), l->gcx,
 	       l->window_p, l->npoint, CoordModeOrigin);
 
@@ -559,6 +639,7 @@ releaseLassoBand(Widget w, LocalInfo * l, XButtonEvent * event,
 	l->pixBox.height += l->pixBox.y;
 	l->pixBox.y = 0;
     }
+
     *mask = XCreatePixmap(XtDisplay(w), XtWindow(w),
 			  l->pixBox.width, l->pixBox.height, 1);
     gc = XCreateGC(XtDisplay(w), *mask, 0, 0);
@@ -582,7 +663,7 @@ static void
 releasePolyBand(Widget w, LocalInfo * l, XButtonEvent * event,
 		OpInfo * info)
 {
-    if (l->first) {
+    if (l->first) {       	
 	l->first = False;
 	return;
     } else {
@@ -616,6 +697,7 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     /*
     **  Check to make sure all buttons are up, before doing this
      */
+    if (event->button >= Button4) return;      
     mask = AllButtonsMask;
     switch (event->button) {
     case Button1:
@@ -636,6 +718,7 @@ release(Widget w, LocalInfo * l, XButtonEvent * event, OpInfo * info)
     }
     if ((event->state & mask) != 0)
 	return;
+    if (event->button == Button3) return;
 
     mpix = None;
 
@@ -666,6 +749,7 @@ SelectBoxAdd(Widget w)
 
     l->areaType = 0;
     l->lastType = 0;
+    l->npoint = -1;
     l->gcx = GetGCX(w);
 
     l->size = 8;
@@ -676,12 +760,14 @@ SelectBoxAdd(Widget w)
 
     OpAddEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
 		      (OpEventProc) press, l);
-    OpAddEventHandler(w, opWindow, ButtonMotionMask, FALSE,
+    OpAddEventHandler(w, opWindow, PointerMotionMask, FALSE,
 		      (OpEventProc) motion, l);
     OpAddEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 		      (OpEventProc) release, l);
     OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
 		      (OpEventProc) keyPress, l);
+    OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+		      (OpEventProc) keyRelease, l);
     SetCrossHairCursor(w);
 
     return l;
@@ -694,12 +780,14 @@ SelectBoxRemove(Widget w, void *p)
 
     OpRemoveEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
 			 (OpEventProc) press, l);
-    OpRemoveEventHandler(w, opWindow, ButtonMotionMask, FALSE,
+    OpRemoveEventHandler(w, opWindow, PointerMotionMask, FALSE,
 			 (OpEventProc) motion, l);
     OpRemoveEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 			 (OpEventProc) release, l);
     OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
 			 (OpEventProc) keyPress, l);
+    OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+			 (OpEventProc) keyRelease, l);
     PwRegionFinish(w, False);
 
     XtFree((XtPointer) l->window_p);
@@ -713,6 +801,7 @@ LassoAdd(Widget w)
     LocalInfo *l = (LocalInfo *) XtMalloc(sizeof(LocalInfo));
 
     l->areaType = 1;
+    l->npoint = -1;
     l->gcx = GetGCX(w);
 
     XtVaSetValues(w, XtNcompress, False, NULL);
@@ -723,12 +812,14 @@ LassoAdd(Widget w)
 
     OpAddEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
 		      (OpEventProc) press, l);
-    OpAddEventHandler(w, opWindow, ButtonMotionMask, FALSE,
+    OpAddEventHandler(w, opWindow, PointerMotionMask, FALSE,
 		      (OpEventProc) motion, l);
     OpAddEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 		      (OpEventProc) release, l);
     OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
 		      (OpEventProc) keyPress, l);
+    OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+		      (OpEventProc) keyRelease, l);
 
     SetPencilCursor(w);
 
@@ -742,12 +833,14 @@ LassoRemove(Widget w, void *p)
 
     OpRemoveEventHandler(w, opWindow | opPixmap, ButtonPressMask, FALSE,
 			 (OpEventProc) press, l);
-    OpRemoveEventHandler(w, opWindow, ButtonMotionMask, FALSE,
+    OpRemoveEventHandler(w, opWindow, PointerMotionMask, FALSE,
 			 (OpEventProc) motion, l);
     OpRemoveEventHandler(w, opWindow, ButtonReleaseMask, FALSE,
 			 (OpEventProc) release, l);
     OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
 			 (OpEventProc) keyPress, l);
+    OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+			 (OpEventProc) keyRelease, l);
     PwRegionFinish(w, False);
 
     XtFree((XtPointer) l->window_p);
@@ -765,6 +858,7 @@ SelectPolyAdd(Widget w)
     l->drawn = False;
     l->first = True;
     l->tracking = False;
+    l->npoint = -1;
     l->gcx = GetGCX(w);
 
     XtVaSetValues(w, XtNcompress, False, NULL);
@@ -781,7 +875,8 @@ SelectPolyAdd(Widget w)
 		      (OpEventProc) releasePolyBand, l);
     OpAddEventHandler(w, opWindow, KeyPressMask, FALSE,
 		      (OpEventProc) keyPress, l);
-
+    OpAddEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+		      (OpEventProc) keyRelease, l);
     SetPencilCursor(w);
 
     return l;
@@ -800,6 +895,8 @@ SelectPolyRemove(Widget w, void *p)
 			 (OpEventProc) releasePolyBand, l);
     OpRemoveEventHandler(w, opWindow, KeyPressMask, FALSE,
 			 (OpEventProc) keyPress, l);
+    OpRemoveEventHandler(w, opWindow, KeyReleaseMask, FALSE,
+			 (OpEventProc) keyRelease, l);
     PwRegionFinish(w, False);
 
     XtFree((XtPointer) l->window_p);
