@@ -13,7 +13,7 @@
 /* |                                                                   | */
 /* +-------------------------------------------------------------------+ */
 
-/* $Id: xpaint.h,v 1.8 1997/09/03 19:31:07 torsten Exp $ */
+/* $Id: xpaint.h,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
 #if defined(HAVE_PARAM_H)
 #include <sys/param.h>
@@ -45,6 +45,13 @@
 #define NULL 0
 #endif				/* AIXV3 */
 
+extern char *routine;
+
+typedef struct {
+    int x, y;
+    Pixmap sourcepix, pix, mask;
+} RegionData;
+
 extern struct Global_s {
     struct {
 	void *image;
@@ -55,8 +62,19 @@ extern struct Global_s {
     Display *display;
     XtAppContext appContext;
     Boolean timeToDie;
+    Boolean explore;
+    Boolean transparent;
     Time currentTime;
     XVisualInfo vis;
+    int join;  /* line caps and joins */
+    int dashnumber, dashoffset; /* dash parameters */
+    char dashlist[50];
+    Widget toplevel, bar, back, canvas, curpaint, patternshell, brushpopup;
+    int nbrushes;
+    void ** brushes;
+    void * patterninfo;
+    int numregions;
+    RegionData * regiondata;
 } Global;
 
 typedef void *(*OperationFunc) (Widget,...);
@@ -64,6 +82,9 @@ typedef void *(*OperationFunc) (Widget,...);
 typedef void *(*OperationAddProc) (Widget);
 typedef void (*OperationRemoveProc) (Widget, void *);
 typedef OperationFunc Operation_t;
+#ifdef remove	/* remove is a macro in sco 3.2v4 */
+#undef remove
+#endif
 typedef struct {
     OperationAddProc add;
     OperationRemoveProc remove;
@@ -91,6 +112,12 @@ struct Global_s Global;
 #define TFILL_CONE   2
 #define TFILL_SQUARE 3
 
+/*
+**  Value at which zoom starts putting in "white" lines
+**    between pixels
+ */
+#define ZOOM_THRESH 5
+
 #define XYtoRECT(x1,y1,x2,y2,rect)					\
 			(rect)->x = MIN(x1,x2); 			\
 			(rect)->y = MIN(y1,y2);				\
